@@ -1,5 +1,4 @@
-/* $XConsortium: imRm.c /main/10 1995/11/18 16:08:18 kaleb $ */
-/* $XFree86: xc/lib/X11/imRm.c,v 3.1 1996/12/24 02:22:46 dawes Exp $ */
+/* $TOG: imRm.c /main/11 1997/06/22 18:26:20 kaleb $ */
 /******************************************************************
 
 	  Copyright 1990, 1991, 1992,1993, 1994 by FUJITSU LIMITED
@@ -31,6 +30,7 @@ PERFORMANCE OF THIS SOFTWARE.
 			       makoto@sm.sony.co.jp
 
 ******************************************************************/
+/* $XFree86: xc/lib/X11/imRm.c,v 3.1.2.4 1998/05/23 13:38:56 dawes Exp $ */
 
 #include <stdio.h>
 #include <X11/Xlib.h>
@@ -38,6 +38,7 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "Xlcint.h"
 #include "Ximint.h"
 #include "Xresource.h"
+#include "snprintf.h"
 
 #ifndef	isalnum
 #define	isalnum(c)	\
@@ -67,26 +68,23 @@ typedef struct _XimValueOffsetInfo {
 			 );
 } XimValueOffsetInfoRec, *XimValueOffsetInfo;
 
-void
-_XimGetResourceName(im, res_name, res_class)
+static void
+_XimGetResourceName(im, res_name, res_class, size)
     Xim		 im;
     char	*res_name;
     char	*res_class;
+    int		size;
 {
     if(im->core.res_name == NULL) {
-	strcpy(res_name, "*");
+	strcpy(res_name, "*xim.");
     } else {
-	strcpy(res_name, im->core.res_name);
-	strcat(res_name, ".");
+	_XSnprintf(res_name, size, "%s.xim.", im->core.res_name);
     }
     if(im->core.res_class == NULL) {
-	strcpy(res_class, "*");
+	strcpy(res_class, "*Xim.");
     } else {
-	strcpy(res_class, im->core.res_class);
-	strcat(res_class, ".");
+	_XSnprintf(res_class, size, "%s.Xim.", im->core.res_class);
     }
-    strcat(res_name, "xim.");
-    strcat(res_class, "Xim.");
 }
 
 #ifdef XIM_CONNECTABLE
@@ -118,7 +116,7 @@ _XimSetProtoResource(im)
     if (!im->core.rdb)
 	return;
 
-    _XimGetResourceName(im, xim_res_name, xim_res_class);
+    _XimGetResourceName(im, xim_res_name, xim_res_class, 256);
 
     _XSnprintf(res_name, sizeof(res_name), "%s%s", xim_res_name, "useAuth");
     _XSnprintf(res_class, sizeof(res_class), "%s%s", xim_res_class, "UseAuth");
@@ -2927,9 +2925,11 @@ _XimDecodeLocalICAttr(res, top, val, mode)
     if(mode & XIM_PREEDIT_ATTR) {
 	info = ic_pre_attr_info;
 	num  = XIMNumber(ic_pre_attr_info);
+	top = (XPointer) &((XimDefICValues *)top)->preedit_attr;
     } else if(mode & XIM_STATUS_ATTR) {
 	info = ic_sts_attr_info;
 	num  = XIMNumber(ic_sts_attr_info);
+	top = (XPointer) &((XimDefICValues *)top)->status_attr;
     } else {
 	info = ic_attr_info;
 	num  = XIMNumber(ic_attr_info);
