@@ -31,7 +31,7 @@
  * Modifier: Takanori Tateno   FUJITSU LIMITED
  *
  */
-/* $XFree86: xc/lib/X11/lcCT.c,v 3.5.2.1 1997/05/03 09:43:42 dawes Exp $ */
+/* $XFree86: xc/lib/X11/lcCT.c,v 3.5.2.3 1998/05/19 02:55:11 dawes Exp $ */
 
 #include "Xlibint.h"
 #include "XlcPubI.h"
@@ -327,7 +327,7 @@ done:
 
 XlcCharSet
 _XlcAddCT(name, encoding)
-    char *name;
+    _Xconst char *name;
     char *encoding;
 {
     CTInfo ct_info;
@@ -402,6 +402,9 @@ _XlcParseCharSet(charset)
     if (charset->ct_sequence == NULL)
 	return False;
 
+    if (charset->name && strlen(charset->name) >= sizeof(buf))
+	return False;
+
     ptr = charset->ct_sequence;
     length = strlen(ptr);
 
