@@ -1,5 +1,4 @@
-/* $XConsortium: lcDB.c /main/11 1996/12/05 11:10:38 swick $ */
-/* $XFree86: xc/lib/X11/lcDB.c,v 3.3 1996/12/23 05:59:59 dawes Exp $ */
+/* $TOG: lcDB.c /main/12 1997/06/02 17:27:50 kaleb $ */
 /*
  *
  * Copyright IBM Corporation 1993
@@ -29,6 +28,10 @@
  *  This is source code modified by FUJITSU LIMITED under the Joint
  *  Development Agreement for the CDE/Motif PST.
  */
+/* $XFree86: xc/lib/X11/lcDB.c,v 3.3.2.3 1998/05/19 02:55:12 dawes Exp $ */
+
+
+
 #ifndef	NOT_X_ENV
 
 #include <X11/Xlib.h>
@@ -398,10 +401,14 @@ get_token(str)
     }
 }
 
+#define CHECKWORD(n) \
+	if (w - word + (n) >= size - 1) {*word = 0; return 0;} else
+
 static int
-get_word(str, word)
+get_word(str, word, size)
     char *str;
     char *word;
+    int size;
 {
     char *p = str, *w = word;
     Token token;
@@ -421,6 +428,7 @@ get_word(str, word)
 		 token != T_DEFAULT){
 	    break;
 	}
+	CHECKWORD(token_len);
 	strncpy(w, p, token_len);
 	p += token_len; w += token_len;
     }
@@ -429,9 +437,10 @@ get_word(str, word)
 }
 
 static int
-get_quoted_word(str, word)
+get_quoted_word(str, word, size)
     char *str;
     char *word;
+    int size;
 {
     char *p = str, *w = word;
     Token token;
@@ -455,6 +464,7 @@ get_quoted_word(str, word)
 	    token = get_token(p);
 	    token_len = token_tbl[token].len;
 	}
+	CHECKWORD(token_len);
 	strncpy(w, p, token_len);
 	p += token_len; w += token_len;
     }
@@ -538,8 +548,9 @@ append_value_list()
 }
 
 static int 
-construct_name(name)
+construct_name(name, size)
     char *name;
+    int size;
 {
     register int i, len = 0;
     char *p = name;
@@ -547,6 +558,8 @@ construct_name(name)
     for(i = 0; i <= parse_info.nest_depth; ++i){
 	len += strlen(parse_info.name[i]) + 1;
     }
+    if (len >= size)
+	return 0;
 
     strcpy(p, parse_info.name[0]);
     p += strlen(parse_info.name[0]);
@@ -587,7 +600,7 @@ store_to_database(db)
     }
     strcpy(new->category, parse_info.category);
 
-    if(! construct_name(name)){
+    if(! construct_name(name, sizeof(name))){
 	goto err;
     }
     new->name = (char *)Xmalloc(strlen(name) + 1);
@@ -822,7 +835,7 @@ f_double_quote(str, token, db)
 	goto err;
     case S_NAME:
     case S_VALUE:
-	len = get_quoted_word(str, word);
+	len = get_quoted_word(str, word, sizeof(word));
 	if(len < 1){
 	    goto err;
 	}
@@ -872,7 +885,7 @@ f_numeric(str, token, db)
     case S_VALUE:
 	token_len = token_tbl[token].len;
 	p = str + token_len;
-	len = get_word(p, word);
+	len = get_word(p, word, sizeof(word));
 	if(len < 1){
 	    goto err;
 	}
@@ -905,7 +918,7 @@ f_default(str, token, db)
     char word[BUFSIZE], *p;
     int len;
 
-    len = get_word(str, word);
+    len = get_word(str, word, sizeof(word));
     if(len < 1){
 	goto err;
     }
@@ -1214,30 +1227,30 @@ _XlcCreateLocaleDataBase(lcd)
     Database p, database = (Database)NULL;
     XlcDatabase lc_db = (XlcDatabase)NULL;
     XrmQuark name_q;
-    char pathname[256], *name;
+    char *name;
     int i, n;
 
     name = _XlcFileName(lcd, "locale");
     if(name == NULL){
 	return (XPointer)NULL;
     }
-#ifdef __EMX__
-    strcpy(pathname,__XOS2RedirRoot(name));
-#else
-    strcpy(pathname, name);
-#endif
-    Xfree(name);
 
-    name_q = XrmStringToQuark(pathname);
+#ifndef __EMX__
+    name_q = XrmStringToQuark(name);
+#else
+    name_q = XrmStringToQuark((char*)__XOS2RedirRoot(name));
+#endif
     for(list = _db_list; list; list = list->next){
 	if(name_q == list->name_q){
 	    list->ref_count++;
+	    Xfree (name);
 	    return XLC_PUBLIC(lcd, xlocale_db) = (XPointer)list->lc_db;
 	}
     }
 
-    database = CreateDatabase(pathname);
+    database = CreateDatabase(name);
     if(database == (Database)NULL){
+	Xfree (name);
 	return (XPointer)NULL;
     }
     n = CountDatabase(database);
@@ -1263,13 +1276,15 @@ _XlcCreateLocaleDataBase(lcd)
     new->next = _db_list;
     _db_list = new;
 
+    Xfree (name);
     return XLC_PUBLIC(lcd, xlocale_db) = (XPointer)lc_db;
 
- err:;
+ err:
     DestroyDatabase(database);
     if(lc_db != (XlcDatabase)NULL){
 	Xfree((char *)lc_db);
     }
+    Xfree (name);
     return (XPointer)NULL;
 }
 
