@@ -1,5 +1,5 @@
-/* $XConsortium: osPexFont.c,v 5.8 94/04/17 20:36:01 rws Exp $ */
-/* $XFree86: xc/programs/Xserver/PEX5/ospex/osPexFont.c,v 3.3 1996/08/11 12:34:10 dawes Exp $ */
+/* $XConsortium: osPexFont.c /main/10 1996/12/06 11:02:43 lehors $ */
+/* $XFree86: xc/programs/Xserver/PEX5/ospex/osPexFont.c,v 3.4.2.4 1998/07/12 18:45:53 robin Exp $ */
 
 /*
 
@@ -49,6 +49,9 @@ SOFTWARE.
 
 */
 
+#ifdef WIN32
+#define _WILLWINSOCK_
+#endif
 #include <X11/Xos.h>
 #ifndef X_NOT_STDC_ENV
 #include <stdlib.h>
@@ -67,6 +70,32 @@ extern char *getenv();
 #define PEX_DEFAULT_FONTPATH "/usr/lib/X11/fonts/PEX"
 #endif
 
+#ifndef X_NOT_POSIX
+#ifdef _POSIX_SOURCE
+#include <limits.h>
+#else
+#define _POSIX_SOURCE
+#include <limits.h>
+#undef _POSIX_SOURCE
+#endif
+#endif /* X_NOT_POSIX */
+#ifndef PATH_MAX
+#ifdef WIN32
+#define PATH_MAX 512
+#else
+#include <sys/param.h>
+#endif
+#ifndef PATH_MAX
+#ifdef MAXPATHLEN
+#define PATH_MAX MAXPATHLEN
+#else
+#define PATH_MAX 1024
+#endif
+#endif
+#endif /* PATH_MAX */
+
+#ifndef WIN32
+
 #ifndef X_NOT_POSIX
 #include <dirent.h>
 #else
@@ -84,9 +113,20 @@ extern char *getenv();
 #endif
 #endif
 
-
 /* A convenient shorthand. */
 typedef struct dirent	 ENTRY;
+#define FileName(file) file->d_name
+
+#else  /* WIN32 */
+
+#define BOOL wBOOL
+#define ATOM wATOM
+#include <windows.h>
+#undef BOOL
+#undef ATOM
+#define FileName(file) file.cFileName
+
+#endif
 
 extern void CopyISOLatin1Lowered();
 extern int get_lowered_truncated_entry();
@@ -263,27 +303,51 @@ ddUSHORT   maxNames;		/* in */
 ddULONG   *numNames;		/* out - number of names found */
 char    ***names;		/* out - pointer to list of strings */
 {
+#ifdef WIN32
+    HANDLE		fontdirh;
+    WIN32_FIND_DATA	dir_entry;
+    char	        path[MAX_PATH];
+#else
     DIR		    *fontdir;
     ENTRY           *dir_entry;
-    char	     pattern[100];
-    char	     entry[100];
+#endif
+    char	     *pattern;
+    char	     entry[PATH_MAX+1];
     int		     i, head, tail, len, total = 0;
     
+    if (!(pattern = (char *)xalloc((unsigned long)(1 + patLen))))
+	return 0;
+
     CopyISOLatin1Lowered((unsigned char*)pattern, pPattern, patLen);
     
     if (!(*names = (char **)xalloc((unsigned long)(ABSOLUTE_MAX_NAMES * sizeof(char *)))))
 	return 0;
     
-    if (!(fontdir = opendir(pex_get_font_directory_path())))
+#ifdef WIN32
+    sprintf(path, "%s/*.*", pex_get_font_directory_path());
+    if ((fontdirh = FindFirstFile(path, &dir_entry)) == INVALID_HANDLE_VALUE) {
+	xfree(*names);
+	xfree(pattern);
 	return 0;
+    }
+#else
+    if (!(fontdir = opendir(pex_get_font_directory_path()))) {
+	xfree(*names);
+	xfree(pattern);
+	return 0;
+    }
+#endif
 
     pex_setup_wild_match(pattern, &head, &tail, &len);
     
-    do {
-	dir_entry = readdir(fontdir);
-	if (dir_entry) {
+#ifdef WIN32
+    do
+#else
+    while (total < maxNames && (dir_entry = readdir(fontdir)))
+#endif
+	{
 	
-	    if (!get_lowered_truncated_entry(dir_entry->d_name, entry))
+	    if (!get_lowered_truncated_entry(FileName(dir_entry), entry))
 		continue;
 
 	    if (pex_is_matching(entry, pattern, head, tail, len) > 0) {
@@ -292,6 +356,7 @@ char    ***names;		/* out - pointer to list of strings */
 		    for (i = 0; i < total; i++)
 			xfree((*names)[i]);
 		    xfree(*names);
+		    xfree(pattern);
 		    return 0;
 		}
 		
@@ -299,9 +364,15 @@ char    ***names;		/* out - pointer to list of strings */
 		total++;
 	    }
 	}
-    } while (dir_entry && total < maxNames);
+#ifdef WIN32
+    while (total < maxNames && FindNextFile(fontdirh, &dir_entry));
+#endif
 
+#ifdef WIN32
+    FindClose(fontdirh);
+#else
     closedir(fontdir);
+#endif
     
     *numNames = total;
     
@@ -374,51 +445,76 @@ LoadPEXFontFile(length, fontname, pFont)
     char *	    fontname;
     diFontHandle    pFont;
 {
-    char                fname[100];
+    char                fname[FILENAME_MAX+1];
     FILE               *fp;
     Font_file_header    header;
     Property           *properties = 0;
     Dispatch           *table = 0;
     miFontHeader       *font = (miFontHeader *)(pFont->deviceData);
     int                 found_first, found_it = 0, err = Success, numChars, np;
-    char		name_to_match[100];
-    char		lowered_entry[100];
+    char		*name_to_match;
+    char		lowered_entry[PATH_MAX+1];
+#ifdef WIN32
+    HANDLE		fontdirh;
+    WIN32_FIND_DATA	dir_entry;
+    char	        path[MAX_PATH];
+#else
     DIR		       *fontdir;
     ENTRY              *dir_entry;
+#endif
     register int        i;
     register Ch_stroke_data **ch_font, *ch_stroke = 0;
     register Dispatch  *tblptr = 0;
     register Property  *propptr = 0;
     register pexFontProp *fpptr = 0;
 
+    if (!(name_to_match = (char *)xalloc((unsigned int)(1 + length))))
+	return (PEXERR(PEXFontError));
+
     CopyISOLatin1Lowered((unsigned char *)name_to_match,
 			 (unsigned char *)fontname, length);
 
     /* open up the font directory and look for matching file names */
+#ifdef WIN32
+    sprintf(path, "%s/*.*", pex_get_font_directory_path());
+    if ((fontdirh = FindFirstFile(path, &dir_entry)) == INVALID_HANDLE_VALUE)
+	return (PEXERR(PEXFontError));
+#else
     if (!(fontdir = opendir(pex_get_font_directory_path())))
 	return (PEXERR(PEXFontError));
+#endif
 
-    do {
-	dir_entry = readdir(fontdir);
-	if (dir_entry) {
+#ifdef WIN32
+    do
+#else
+    while(!found_it && (dir_entry = readdir(fontdir)))
+#endif
+	{
 	    /* strip off .phont and make all lower case */
-	    if (!get_lowered_truncated_entry(dir_entry->d_name, lowered_entry))
+	    if (!get_lowered_truncated_entry(FileName(dir_entry), lowered_entry))
 		continue;
 		
 	    /* does this match what got passed in? */
 	    if (strcmp(lowered_entry, name_to_match) == 0)
 		found_it = 1;
 	}
-    } while (dir_entry && !found_it);
+    xfree(name_to_match);
+#ifdef WIN32
+    while (!found_it && FindNextFile(fontdirh, &dir_entry) && !found_it);
+#endif
     
     if (!found_it)
 	return (PEXERR(PEXFontError));
     
     (void) strcpy(fname, pex_get_font_directory_path());
     (void) strcat(fname, "/");
-    (void) strcat(fname, dir_entry->d_name);
+    (void) strcat(fname, FileName(dir_entry));
     
+#ifdef WIN32
+    FindClose(fontdirh);
+#else
     closedir(fontdir);
+#endif
 
     if ((fp = fopen(fname, "r")) == NULL)
 	return (PEXERR(PEXFontError));
