@@ -26,7 +26,7 @@
  *
  */
 
-/* $XFree86: xc/programs/Xserver/hw/xfree86/SuperProbe/Main.c,v 3.17.2.4 1997/05/18 12:00:05 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/SuperProbe/Main.c,v 3.17.2.6 1998/02/15 23:31:54 robin Exp $ */
 
 #include "Probe.h"
 #include "PatchLevel.h"
@@ -77,6 +77,7 @@ static Chip_Descriptor *SVGA_Descriptors[] = {
     &ARK_Descriptor,
     &Alliance_Descriptor,
     &SigmaDesigns_Descriptor,
+    &Intergraphics_Descriptor,
     &CT_Descriptor, /* I think this is screwing people up, so put it last */
     NULL
 };
@@ -356,7 +357,7 @@ int argc;
 char *argv[];
 {
     char *p, *order = NULL;
-    Byte copyright[3];
+    Byte copyright[5];
     Word Ports[10];		/* For whatever we need */
     int Primary = -1;
     int Secondary = -1;
@@ -623,7 +624,7 @@ char *argv[];
          * Look for 'IBM' at Bios_Base+0x1E of the BIOS.  It will be there for
          * an EGA or VGA.
          */
-        if (ReadBIOS(COPYRIGHT_BASE, copyright, 3) < 0)
+        if (ReadBIOS(COPYRIGHT_BASE, copyright, 5) < 0)
         {
     	    fprintf(stderr, "%s: Failed to read BIOS\n", MyName);
     	    CloseVideo();
@@ -635,7 +636,10 @@ char *argv[];
 	((NoBIOS) ||
          ((copyright[0] == 'I') && 
           (copyright[1] == 'B') &&
-          (copyright[2] == 'M')))))
+          (copyright[2] == 'M')) ||
+         ((copyright[2] == 'P') && 
+          (copyright[3] == 'C') &&
+          (copyright[4] == 'I')))))
     {
     	/*
     	 * It's an EGA or VGA
