@@ -26,7 +26,7 @@
  *
  */
 
-/* $XFree86: xc/programs/Xserver/hw/xfree86/SuperProbe/Print.c,v 3.46.2.4 1997/05/22 14:00:34 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/SuperProbe/Print.c,v 3.46.2.10 1998/07/16 06:54:54 hohndel Exp $ */
 
 #include "Probe.h"
 
@@ -42,9 +42,11 @@ static CONST char *SVGA_Names[NUM_VENDORS+1][CHPS_PER_VENDOR] =
 		  "ATI 88800GX-C", "ATI 88800GX-D", "ATI 88800GX-E",
 		  "ATI 88800GX-F",
 		  "ATI 88800CX",
-		  "ATI 264CT", "ATI 264ET", "ATI 264LT",
+		  "ATI 264CT", "ATI 264ET",
 		  "ATI 264VT", "ATI 264VT-B",
-		  "ATI 264GT (3D Rage I)", "ATI 264GT-B (3D Rage II)" },
+		  "ATI 264GT (3D Rage I)", "ATI 264GT-B (3D Rage II or II+)",
+		  "ATI 264LT",
+		  "ATI 264GT3 (3D Rage III)" },
 /* AL */	{ "Avance Logic (chipset unknown)",
 		  "Avance Logic 2101", "Avance Logic 2228" },
 /* CT */	{ "Chips & Tech (chipset unknown)",
@@ -55,7 +57,8 @@ static CONST char *SVGA_Names[NUM_VENDORS+1][CHPS_PER_VENDOR] =
 		  "Chips & Tech F65520", "Chips & Tech F65530",
 		  "Chips & Tech F65540", "Chips & Tech F65545",
 		  "Chips & Tech F65548", "Chips & Tech F65546",
-		  "Chips & Tech F65550", "Chips & Tech F65554"},
+		  "Chips & Tech F65550", "Chips & Tech F65554",
+		  "Chips & Tech F65555", "Chips & Tech F68554" },
 /* Cirrus */	{ "Cirrus (chipset unknown)",
 		  "Cirrus CL-GD 510/520", "Cirrus CL-GD 610/620",
 		  "Cirrus Video7 OEM",
@@ -128,6 +131,9 @@ static CONST char *SVGA_Names[NUM_VENDORS+1][CHPS_PER_VENDOR] =
 		  "S3 ViRGE/DX",
 		  "S3 ViRGE/GX",
 		  "S3 PLATO/PX",
+		  "S3 ViRGE/GX2",
+		  "S3 ViRGE/MX",
+		  "S3 ViRGE/MXP",
 	       },
 /* Trident */	{ "Trident (chipset unknown)",
 		  "Trident LX8200",
@@ -150,7 +156,7 @@ static CONST char *SVGA_Names[NUM_VENDORS+1][CHPS_PER_VENDOR] =
 		  "Tseng ET4000/W32p Rev B",
 		  "Tseng ET4000/W32p Rev C",
 		  "Tseng ET4000/W32p Rev D",
-		  "Tseng ET6000",
+		  "Tseng ET6000", "Tseng ET6100", "Tseng ET6300"
 		},
 /* UMC */	{ "UMC 85c408" },
 /* Video7 */	{ "Video7 (chipset unknown)",
@@ -187,9 +193,13 @@ static CONST char *SVGA_Names[NUM_VENDORS+1][CHPS_PER_VENDOR] =
 /* Matrox */	{ "Matrox (chipset unknown)",
 		  "Matrox Atlas",
 		  "Matrox Millennium",
-		  "Matrox Mystique" },
+		  "Matrox Mystique",
+		  "Matrox Millennium II" },
 /* Sigma Designs */	{ "Sigma Designs (chipset unknown)",
 		  "Sigma Designs REALmagic64/GX (SD 6425)" },
+/* Intergraphcis */	{ "Intergraphics (chipset unknown)",
+		  "Intergraphics IGA-1680",
+		  "Intergraphics IGA-1682" },
 };
 
 static CONST char *Herc_Names[] = 
