@@ -27,7 +27,7 @@
  * 
  */
 
-/* $XFree86: xc/programs/Xserver/hw/xfree86/accel/i128/i128misc.c,v 3.5.2.1 1997/05/22 14:00:37 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/accel/i128/i128misc.c,v 3.5.2.4 1998/01/12 03:02:09 robin Exp $ */
 
 #include "servermd.h"
 
@@ -52,7 +52,7 @@ extern pointer vgaBase;
 extern pointer i128VideoMem;
 extern struct i128mem i128mem;
 
-extern Bool xf86Exiting, xf86Resetting, xf86ProbeFailed, xf86Verbose;
+extern Bool xf86Exiting, xf86Resetting;
 
 static Bool AlreadyInited = FALSE;
 static Bool i128ModeSwitched = FALSE;
@@ -189,13 +189,9 @@ i128EnterLeaveVT(enter, screen_idx)
 
 	 if ((pointer)pspix->devPrivate.ptr != i128VideoMem && ppix) {
 	    pspix->devPrivate.ptr = i128VideoMem;
-#ifdef WORKWORKWORK
-	    (*i128ImageWriteFunc)(0, 0, pScreen->width, pScreen->height,
-			        ppix->devPrivate.ptr,
-			        PixmapBytePad(i128DisplayWidth, 
-					      pScreen->rootDepth),
-			        0, 0, i128alu[GXcopy], ~0);
-#endif
+	    memcpy((char *)i128VideoMem, (char *)ppix->devPrivate.ptr,
+		   pScreen->height *
+		   PixmapBytePad(i128DisplayWidth, pScreen->rootDepth));
 	 }
       }
       if (ppix) {
@@ -207,13 +203,9 @@ i128EnterLeaveVT(enter, screen_idx)
 	 ppix = (pScreen->CreatePixmap)(pScreen, i128DisplayWidth,
 					pScreen->height, pScreen->rootDepth);
 	 if (ppix) {
-#ifdef WORKWORKWORK
-	    (*i128ImageReadFunc)(0, 0, pScreen->width, pScreen->height,
-			       ppix->devPrivate.ptr,
-			       PixmapBytePad(i128DisplayWidth, 
-					     pScreen->rootDepth),
-			       0, 0, ~0);
-#endif
+	    memcpy((char *)ppix->devPrivate.ptr, (char *)i128VideoMem,
+		   pScreen->height *
+		   PixmapBytePad(i128DisplayWidth, pScreen->rootDepth));
 	    pspix->devPrivate.ptr = ppix->devPrivate.ptr;
 	 }
       }
@@ -273,10 +265,11 @@ i128SaveScreen(pScreen, on)
       /* the server is running on the current vt */
       /* so just go for it */
 
-      if (on)
-         i128mem.rbase_g[0x58/4] |= 0x40;
-      else
-         i128mem.rbase_g[0x58/4] &= ~0x40;
+      if (on) {
+         i128mem.rbase_g[CRT_1CON] |= 0x40;				MB;
+      } else {
+         i128mem.rbase_g[CRT_1CON] &= ~0x40;				MB;
+      }
    }
    return (TRUE);
 }
@@ -316,7 +309,7 @@ i128AdjustFrame(int x, int y)
 {
    int   Base;
    unsigned char tmp;
-   extern int i128HDisplay, i128AdjustCursorXPos;
+   extern int i128HDisplay, i128AdjustCursorXPos, i128DisplayOffset;
 #define I128_PAN_MASK 0x01FFFFE0
 
    if (OFLG_ISSET(OPTION_SHOWCACHE, &i128InfoRec.options)) {
@@ -328,7 +321,7 @@ i128AdjustFrame(int x, int y)
       x  = i128DisplayWidth - i128HDisplay;
 
    Base = ((y*i128DisplayWidth + x) * (i128InfoRec.bitsPerPixel/8));
-   i128mem.rbase_g[DB_ADR] = Base & I128_PAN_MASK;
+   i128mem.rbase_g[DB_ADR] = (Base & I128_PAN_MASK) + i128DisplayOffset; MB;
 
    /* now warp the cursor after the screen move */
    i128AdjustCursorXPos = Base - (Base & I128_PAN_MASK);
