@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/accel/s3/s3.c,v 3.155.2.10 1997/06/01 12:33:29 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/accel/s3/s3.c,v 3.155.2.18 1998/07/16 06:54:58 hohndel Exp $ */
 /*
  * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
  * 
@@ -59,7 +59,6 @@
 #endif
 
 #ifdef PC98
-#include "pc98_vers.h"
 #include "s3pc98.h"
 #endif
 
@@ -153,7 +152,8 @@ ScrnInfoRec s3InfoRec =
    0,				/* int textClockFreq */
    NULL,                        /* char* DCConfig */
    NULL,                        /* char* DCOptions */
-   0				/* int MemClk */
+   0,				/* int MemClk */
+   0				/* int LCDClk */
 #ifdef XFreeXDGA
    ,0,				/* int directMode */
    s3SetVidPage,		/* Set Vid Page */
@@ -382,9 +382,6 @@ s3PrintIdent()
       c += strlen(id);
     }
   ErrorF("\n");
-#ifdef PC98
-  ErrorF("  PC98: Supported Video Boards:\n\t%s\n",PC98_S3_BOARDS);
-#endif
 }
 
 
@@ -472,6 +469,36 @@ s3GetPCIInfo()
       return NULL;
 
    while ((pcrp = pcrpp[i])) {
+      /* check for fake S3 chips :-(( */
+      if (pcrp->_vendor == PCI_VENDOR_SIGMADESIGNS ||
+	  pcrp->_vendor == PCI_VENDOR_INTERGRAPHICS) {
+	char *vendor, chip[80];
+	if (pcrp->_vendor == PCI_VENDOR_SIGMADESIGNS) {
+	  vendor = "Sigma Designs";
+	  if (pcrp->_device == PCI_CHIP_SD_REALMAGIG64GX)
+	    strcpy(chip,"REALmagic64/GX (SD 6425) chip");
+	  else
+	    sprintf(chip,"unknown chip (chip_id 0x%x)",pcrp->_device);
+	}
+	else {
+	  vendor = "Intergraphics";
+	  if (pcrp->_vendor == PCI_CHIP_INTERG_1680)
+	    strcpy(chip,"IGA-1680 chip");
+	  else if (pcrp->_vendor == PCI_CHIP_INTERG_1682)
+	    strcpy(chip,"IGA-1682 chip");
+	  else
+	    sprintf(chip,"unknown chip (chip_id 0x%x)",pcrp->_device);
+	}
+	ErrorF("\n%s %s: WARNING: %s %s detected!\n"
+	   "\tNote: this chip is not a product of S3, Inc., and it is not\n"
+	   "\tcompatible with the XFree86 S3 drivers.  We understand that\n"
+	   "\tsome video cards are being sold with these chips relabeled\n"
+	   "\tas S3 Inc. chips, including S3's logo.  They are NOT S3 chips.\n"
+	   "\tPlease see http://www.s3.com\n\n"
+	       ,XCONFIG_PROBED, s3InfoRec.name
+	       ,vendor, chip);
+      }
+
       if (pcrp->_vendor == PCI_S3_VENDOR_ID) {
 	 found = TRUE;
 	 switch (pcrp->_device) {
@@ -565,25 +592,18 @@ s3GetPCIInfo()
       for (j=0; (pcrp = pcrpp[j]); j++) {
 	 if (i != j) {
 	    map_64m[ (pcrp->_base0 >> 26) & 0x3f] = 1;
-	    map_64m[((pcrp->_base0+0x3ffffff) >> 26) & 0x3f] = 1;
 	    map_64m[ (pcrp->_base1 >> 26) & 0x3f] = 1;
-	    map_64m[((pcrp->_base1+0x3ffffff) >> 26) & 0x3f] = 1;
 	    map_64m[ (pcrp->_base2 >> 26) & 0x3f] = 1;
-	    map_64m[((pcrp->_base2+0x3ffffff) >> 26) & 0x3f] = 1;
 	    map_64m[ (pcrp->_base3 >> 26) & 0x3f] = 1;
-	    map_64m[((pcrp->_base3+0x3ffffff) >> 26) & 0x3f] = 1;
 	    map_64m[ (pcrp->_base4 >> 26) & 0x3f] = 1;
-	    map_64m[((pcrp->_base4+0x3ffffff) >> 26) & 0x3f] = 1;
 	    map_64m[ (pcrp->_base5 >> 26) & 0x3f] = 1;
-	    map_64m[((pcrp->_base5+0x3ffffff) >> 26) & 0x3f] = 1;
 	 }
       }
 
       /* check for 64MB alignment and free space */
       
       if ((base0 & 0x3ffffff) ||
-	  map_64m[(base0 >> 26) & 0x3f] || 
-	  map_64m[((base0+0x3ffffff) >> 26) & 0x3f]) {
+	  map_64m[(base0 >> 26) & 0x3f]) {
 	 for (j=63; j>=16 && map_64m[j]; j--);
 	 info.MemBase = ((unsigned long)j) << 26;
 	 ErrorF("%s %s: PCI: base address not correctly aligned or address conflict\n",
@@ -857,9 +877,9 @@ s3Probe()
 	 } else if (S3_TRIO32_SERIES(s3ChipId)) {
 	    chipname = "Trio32";
 	 } else if (S3_TRIO64UVP_SERIES(s3ChipId)) {
-	    chipname = "Trio64UV+ (preliminary support; please report)";
+	    chipname = "Trio64UV+";
 	 } else if (S3_AURORA64VP_SERIES(s3ChipId)) {
-	    chipname = "Aurora64V+ (preliminary support; please report)";
+	    chipname = "Aurora64V+";
 	 } else if (S3_TRIO64V_SERIES(s3ChipId /* , s3ChipRev */)) {
 	    chipname = "Trio64V+";
 	 } else if (S3_TRIO64V2_SERIES(s3ChipId)) {
@@ -1008,6 +1028,10 @@ s3Probe()
    OFLG_SET(OPTION_ELSA_W2000PRO_X8, &validOptions);
    OFLG_SET(OPTION_MIRO_80SV, &validOptions);
    OFLG_SET(OPTION_NO_PCI_DISC, &validOptions);
+   OFLG_SET(OPTION_NO_SPLIT_XFER, &validOptions);
+   if (S3_AURORA64VP_SERIES(s3ChipId))
+      OFLG_SET(OPTION_LCD_CENTER, &validOptions);
+      
    xf86VerifyOptions(&validOptions, &s3InfoRec);
 
 #ifdef __alpha__
@@ -1400,7 +1424,7 @@ s3Probe()
      }
    } 
 
-#if 0
+#if 1
    else {
     /* shouldn't we probe it anyway to ensure options are set? */
     if(!(s3Ramdacs[s3RamdacType].DacProbe)()) {
@@ -1598,6 +1622,17 @@ s3Probe()
 	 s3InfoRec.maxClock = 50000;  
    }
 
+   /* override maxClock settings form ramdac code... */
+   if (OFLG_ISSET(XCONFIG_DACSPEED, &s3InfoRec.xconfigFlag))
+      if (xf86bpp > 24 && s3InfoRec.dacSpeeds[3] > 0)
+	 s3InfoRec.maxClock = s3InfoRec.dacSpeeds[3];
+      else if (xf86bpp >= 24 && s3InfoRec.dacSpeeds[2] > 0)
+	 s3InfoRec.maxClock = s3InfoRec.dacSpeeds[2];
+      else if (xf86bpp > 8 && xf86bpp < 24 && s3InfoRec.dacSpeeds[1] > 0)
+	 s3InfoRec.maxClock = s3InfoRec.dacSpeeds[1];
+      else if (xf86bpp <= 8 && s3InfoRec.dacSpeeds[0] > 0)
+	 s3InfoRec.maxClock = s3InfoRec.dacSpeeds[0];
+
    if (xf86Verbose) {
       if (! OFLG_ISSET(CLOCK_OPTION_PROGRAMABLE, &s3InfoRec.clockOptions)) {
 	 for (j = 0; j < s3InfoRec.clocks; j++) {
