@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/accel/s3/s3gc.c,v 3.7 1996/09/01 04:15:33 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/accel/s3/s3gc.c,v 3.9 1997/01/08 20:33:47 dawes Exp $ */
 /*
 
 Copyright (c) 1987  X Consortium
@@ -55,7 +55,7 @@ Modified for the 8514/A by Kevin E. Martin (martin@cs.unc.edu)
  * Modified by Amancio Hasty and Jon Tombs
  * 
  */
-/* $XConsortium: s3gc.c /main/6 1995/11/12 19:06:29 kaleb $ */
+/* $XConsortium: s3gc.c /main/8 1996/10/23 11:44:47 kaleb $ */
 
 #include "X.h"
 #include "Xmd.h"
@@ -82,7 +82,29 @@ Modified for the 8514/A by Kevin E. Martin (martin@cs.unc.edu)
 
 #include "s3.h"
 
-static void s3ValidateGC();
+static void
+s3ValidateGC(
+#if NeedFunctionPrototypes
+    GCPtr	/* pGC */,
+    Mask	/* changes */,
+    DrawablePtr	/* pDrawable */
+#endif
+);
+
+static GCOps *
+matchCommon(
+#if NeedFunctionPrototypes
+     GCPtr	/* pGC */,
+     cfbPrivGCPtr /* devPriv */
+#endif
+);
+
+static GCOps *
+cfbCreateOps(
+#if NeedFunctionPrototypes
+     GCOps *	/* prototype */
+#endif
+);
 
 static GCFuncs cfbFuncs =
 {
@@ -117,7 +139,7 @@ static GCOps s3Ops =
    miImageGlyphBlt,
    miPolyGlyphBlt,
    miPushPixels,
-   NULL,
+   { NULL }
 };
 
 static GCOps cfbTEOps1Rect =
@@ -146,7 +168,7 @@ static GCOps cfbTEOps1Rect =
    cfbTEGlyphBlt8,
    cfbPolyGlyphBlt8,
    cfbPushPixels8,
-   NULL,
+   { NULL },
 };
 
 static GCOps cfbTEOps =
@@ -175,7 +197,7 @@ static GCOps cfbTEOps =
    cfbTEGlyphBlt8,
    cfbPolyGlyphBlt8,
    cfbPushPixels8,
-   NULL,
+   { NULL },
 };
 
 static GCOps cfbNonTEOps1Rect =
@@ -204,7 +226,7 @@ static GCOps cfbNonTEOps1Rect =
    cfbImageGlyphBlt8,
    cfbPolyGlyphBlt8,
    cfbPushPixels8,
-   NULL,
+   { NULL },
 };
 
 static GCOps cfbNonTEOps =
@@ -233,7 +255,7 @@ static GCOps cfbNonTEOps =
    cfbImageGlyphBlt8,
    cfbPolyGlyphBlt8,
    cfbPushPixels8,
-   NULL,
+   { NULL },
 };
 
 static GCOps *
@@ -348,7 +370,7 @@ s3ValidateGC(pGC, changes, pDrawable)
 {
    WindowPtr pWin;
    int   mask;			/* stateChanges */
-   int   index;			/* used for stepping through bitfields */
+   int   indx2;			/* used for stepping through bitfields */
    int   new_rrop;
    int   new_line, new_text, new_fillspans, new_fillarea;
    int   new_rotate;
@@ -484,8 +506,8 @@ s3ValidateGC(pGC, changes, pDrawable)
    }
    mask = changes;
    while (mask) {
-      index = lowbit(mask);
-      mask &= ~index;
+      indx2 = lowbit(mask);
+      mask &= ~indx2;
 
     /*
      * this switch acculmulates a list of which procedures might have to
@@ -495,7 +517,7 @@ s3ValidateGC(pGC, changes, pDrawable)
      * and the font have been changed, or any other pair of items that both
      * change the same thing.
      */
-      switch (index) {
+      switch (indx2) {
 	case GCFunction:
 	case GCForeground:
 	   new_rrop = TRUE;
