@@ -1,5 +1,5 @@
 /*
- * $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Init.c,v 3.66.2.1 1997/05/18 12:00:07 dawes Exp $
+ * $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Init.c,v 3.66.2.3 1998/02/15 16:09:04 hohndel Exp $
  *
  * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
  *
@@ -62,10 +62,6 @@ extern int atoi();
 extern int xtest_command_key;
 #endif /* XTESTEXT1 */
 
-#ifdef PC98
-#include "pc98_vers.h"
-#endif
-
 #ifdef __EMX__
 #define seteuid(x) /*nothing*/
 #define setruid(x) /*nothing*/
@@ -405,6 +401,9 @@ InitInput(argc, argv)
 
 #ifdef XINPUT
   (xf86Info.pMouse)->public.devicePrivate = xf86Info.mouseLocal;
+#if 0 /* Is this needed?? */
+  ((LocalDevicePtr) xf86Info.mouseLocal)->dev = xf86Info.pMouse;
+#endif
 #else
   (xf86Info.pMouse)->public.devicePrivate = (pointer) xf86Info.mouseDev;
 #endif
