@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/vga256/drivers/s3v/s3v_driver.c,v 1.1.2.10 1997/06/01 12:33:40 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/vga256/drivers/s3v/s3v_driver.c,v 1.1.2.18 1998/07/16 06:55:06 hohndel Exp $ */
 
 /*
  *
@@ -151,7 +151,10 @@ SymTabRec s3vChipTable[] = {
    { S3_UNKNOWN,   "unknown"},
    { S3_ViRGE,     "ViRGE"}, 
    { S3_ViRGE_VX,  "ViRGE/VX"},
-   { S3_ViRGE_DXGX,  "ViRGE/DXGX"},
+   { S3_ViRGE_DXGX,"ViRGE/DXGX"},
+   { S3_ViRGE_GX2, "ViRGE/GX2"},
+   { S3_ViRGE_MX,  "ViRGE/MX"},
+   { S3_ViRGE_MXP, "ViRGE/MX+"},
    { -1,           ""},
    };
 
@@ -217,8 +220,8 @@ Bool enter;
       outb(vgaCRIndex, 0x39);    
       outb(vgaCRReg, 0xa5);
       outb(vgaCRIndex, 0x40);
-      tmp = inb(vgaCRIndex);     
-      outb(vgaCRReg, tmp | 0x01);   /* Unlocks extended functions */
+      tmp = inb(vgaCRReg);
+      outb(vgaCRReg, tmp & ~0x01);   /* avoid lockups when reading I/O port 0x92e8 */
       enterCalled = TRUE;
       }
 
@@ -320,6 +323,18 @@ unsigned char tmp, cr3a, cr53, cr66, cr67;
    outb(0x3c5, restore->SR12);
    outb(0x3c4, 0x13);
    outb(0x3c5, restore->SR13);
+   if (S3_ViRGE_GX2_SERIES(s3vPriv.chip) || S3_ViRGE_MX_SERIES(s3vPriv.chip)) {
+     outb(0x3c4, 0x29);
+     outb(0x3c5, restore->SR29);
+     outb(0x3c4, 0x54);
+     outb(0x3c5, restore->SR54);
+     outb(0x3c4, 0x55);
+     outb(0x3c5, restore->SR55);
+     outb(0x3c4, 0x56);
+     outb(0x3c5, restore->SR56);
+     outb(0x3c4, 0x57);
+     outb(0x3c5, restore->SR57);
+   }
    outb(0x3c4, 0x15);
    outb(0x3c5, restore->SR15); 
 
@@ -341,6 +356,8 @@ unsigned char tmp, cr3a, cr53, cr66, cr67;
    outb(vgaCRReg, restore->CR43);
    outb(vgaCRIndex, 0x65);             
    outb(vgaCRReg, restore->CR65);
+   outb(vgaCRIndex, 0x6d);
+   outb(vgaCRReg, restore->CR6D);
 
 
    /* Restore the desired video mode with CR67 */
@@ -348,13 +365,19 @@ unsigned char tmp, cr3a, cr53, cr66, cr67;
    outb(vgaCRIndex, 0x67);             
    cr67 = inb(vgaCRReg) & 0xf; /* Possible hardware bug on VX? */
    outb(vgaCRReg, 0x50 | cr67); 
+   usleep(10000);
+   outb(vgaCRIndex, 0x67);             
    outb(vgaCRReg, restore->CR67 & ~0x0c); /* Don't enable STREAMS yet */
 
    /* Other mode timing and extended regs */
    outb(vgaCRIndex, 0x34);             
    outb(vgaCRReg, restore->CR34);
+   outb(vgaCRIndex, 0x40);             
+   outb(vgaCRReg, restore->CR40);
    outb(vgaCRIndex, 0x42);             
    outb(vgaCRReg, restore->CR42);
+   outb(vgaCRIndex, 0x45);
+   outb(vgaCRReg, restore->CR45);
    outb(vgaCRIndex, 0x51);             
    outb(vgaCRReg, restore->CR51);
    outb(vgaCRIndex, 0x54);             
@@ -373,6 +396,9 @@ unsigned char tmp, cr3a, cr53, cr66, cr67;
    if (s3vPriv.chip == S3_ViRGE_DXGX) {
       outb(vgaCRIndex, 0x86);
       outb(vgaCRReg, restore->CR86);
+   }
+   if (s3vPriv.chip == S3_ViRGE_DXGX || S3_ViRGE_GX2_SERIES(s3vPriv.chip) || 
+       S3_ViRGE_MX_SERIES(s3vPriv.chip)) {
       outb(vgaCRIndex, 0x90);
       outb(vgaCRReg, restore->CR90);
    }
@@ -398,6 +424,18 @@ unsigned char tmp, cr3a, cr53, cr66, cr67;
    outb(0x3c5, restore->SR12);
    outb(0x3c4, 0x13);
    outb(0x3c5, restore->SR13);
+   if (S3_ViRGE_GX2_SERIES(s3vPriv.chip) || S3_ViRGE_MX_SERIES(s3vPriv.chip)) {
+     outb(0x3c4, 0x29);
+     outb(0x3c5, restore->SR29);
+     outb(0x3c4, 0x54);
+     outb(0x3c5, restore->SR54);
+     outb(0x3c4, 0x55);
+     outb(0x3c5, restore->SR55);
+     outb(0x3c4, 0x56);
+     outb(0x3c5, restore->SR56);
+     outb(0x3c4, 0x57);
+     outb(0x3c5, restore->SR57);
+   }
 
    outb(0x3c4, 0x18);
    outb(0x3c5, restore->SR18); 
@@ -417,6 +455,8 @@ unsigned char tmp, cr3a, cr53, cr66, cr67;
    VerticalRetraceWait();
    outb(vgaCRIndex, 0x67);    
    outb(vgaCRReg, 0x50);   /* For possible bug on VX?! */          
+   usleep(10000);
+   outb(vgaCRIndex, 0x67);
    outb(vgaCRReg, restore->CR67); 
 
 
@@ -497,8 +537,32 @@ unsigned char cr3a, cr53, cr66;
     * in the generic VGA portion.
     */
 
+   outb(vgaCRIndex, 0x66);
+   cr66 = inb(vgaCRReg);
+   outb(vgaCRReg, cr66 | 0x80);
+   outb(vgaCRIndex, 0x3a);
+   cr3a = inb(vgaCRReg);
+   outb(vgaCRReg, cr3a | 0x80);
+
+   outb(vgaCRIndex, 0x66);
+   cr66 = inb(vgaCRReg);
+   outb(vgaCRReg, cr66 | 0x80);
+   outb(vgaCRIndex, 0x3a);
+   cr3a = inb(vgaCRReg);
+   outb(vgaCRReg, cr3a | 0x80);
+
    save = (vgaS3VPtr)vgaHWSave((vgaHWPtr)save, sizeof(vgaS3VRec));
 
+   outb(vgaCRIndex, 0x66);
+   outb(vgaCRReg, cr66);
+   outb(vgaCRIndex, 0x3a);             
+   outb(vgaCRReg, cr3a);
+
+   outb(vgaCRIndex, 0x66);
+   outb(vgaCRReg, cr66);
+   outb(vgaCRIndex, 0x3a);             
+   outb(vgaCRReg, cr3a);
+
    /* First unlock extended sequencer regs */
    outb(0x3c4, 0x08);
    outb(0x3c5, 0x06); 
@@ -512,8 +576,12 @@ unsigned char cr3a, cr53, cr66;
    save->CR36 = inb(vgaCRReg);
    outb(vgaCRIndex, 0x3a);             
    save->CR3A = inb(vgaCRReg);
-   outb(vgaCRIndex, 0x42);             
+   outb(vgaCRIndex, 0x40);
+   save->CR40 = inb(vgaCRReg);
+   outb(vgaCRIndex, 0x42);
    save->CR42 = inb(vgaCRReg);
+   outb(vgaCRIndex, 0x45);
+   save->CR45 = inb(vgaCRReg);
    outb(vgaCRIndex, 0x51);             
    save->CR51 = inb(vgaCRReg);
    outb(vgaCRIndex, 0x53);             
@@ -538,6 +606,9 @@ unsigned char cr3a, cr53, cr66;
    if (s3vPriv.chip == S3_ViRGE_DXGX) {
       outb(vgaCRIndex, 0x86);
       save->CR86 = inb(vgaCRReg);
+   }
+   if (s3vPriv.chip == S3_ViRGE_DXGX || S3_ViRGE_GX2_SERIES(s3vPriv.chip) || 
+       S3_ViRGE_MX_SERIES(s3vPriv.chip)) {
       outb(vgaCRIndex, 0x90);
       save->CR90 = inb(vgaCRReg);
    }
@@ -556,6 +627,8 @@ unsigned char cr3a, cr53, cr66;
    save->CR5E = inb(vgaCRReg);  
    outb(vgaCRIndex, 0x65);             
    save->CR65 = inb(vgaCRReg);
+   outb(vgaCRIndex, 0x6d);
+   save->CR6D = inb(vgaCRReg);
 
 
    /* Save sequencer extended regs for DCLK PLL programming */
@@ -569,6 +642,18 @@ unsigned char cr3a, cr53, cr66;
    save->SR12 = inb(0x3c5);
    outb(0x3c4, 0x13);
    save->SR13 = inb(0x3c5);
+   if (S3_ViRGE_GX2_SERIES(s3vPriv.chip) || S3_ViRGE_MX_SERIES(s3vPriv.chip)) {
+     outb(0x3c4, 0x29);
+     save->SR29 = inb(0x3c5);
+     outb(0x3c4, 0x54);
+     save->SR54 = inb(0x3c5);
+     outb(0x3c4, 0x55);
+     save->SR55 = inb(0x3c5);
+     outb(0x3c4, 0x56);
+     save->SR56 = inb(0x3c5);
+     outb(0x3c4, 0x57);
+     save->SR57 = inb(0x3c5);
+   }
 
    outb(0x3c4, 0x15);
    save->SR15 = inb(0x3c5);
@@ -673,6 +758,7 @@ S3VProbe()
 S3PCIInformation *pciInfo = NULL;
 unsigned char config1, config2, m, n, n1, n2, cr66;
 int mclk;
+DisplayModePtr pMode, pEnd;
 
    if (vga256InfoRec.chipset) {
       if (StrCaseCmp(vga256InfoRec.chipset,S3VIdent(0)))
@@ -686,12 +772,15 @@ int mclk;
    if (!pciInfo)
       return FALSE;
 
-   if (pciInfo && pciInfo->MemBase)
+   if (pciInfo && pciInfo->MemBase && !vga256InfoRec.MemBase)
       vga256InfoRec.MemBase = pciInfo->MemBase;
    if (pciInfo)
       if(pciInfo->ChipType != S3_ViRGE && 
          pciInfo->ChipType != S3_ViRGE_VX &&
-	 pciInfo->ChipType != S3_ViRGE_DXGX){
+	 pciInfo->ChipType != S3_ViRGE_DXGX &&
+	 pciInfo->ChipType != S3_ViRGE_GX2 &&
+	 pciInfo->ChipType != S3_ViRGE_MX &&
+	 pciInfo->ChipType != S3_ViRGE_MXP){
           if (xf86Verbose > 1)
              ErrorF("%s %s: Unsupported (non-ViRGE) S3 chipset detected!\n", 
                 XCONFIG_PROBED, vga256InfoRec.name);
@@ -704,7 +793,7 @@ int mclk;
          ErrorF("%s %s: using driver for chipset \"%s\"\n",XCONFIG_PROBED, 
             vga256InfoRec.name, S3VIdent(0));
 	 }
-   
+
    vga256InfoRec.chipset = S3VIdent(0);
 
 #ifdef __alpha__
@@ -761,6 +850,16 @@ int mclk;
          }
          vga256InfoRec.videoRam -= s3vPriv.MemOffScreen;
       }
+      else if (S3_ViRGE_GX2_SERIES(s3vPriv.chip) || S3_ViRGE_MX_SERIES(s3vPriv.chip)) {
+         switch((config1 & 0xC0) >> 6) {
+         case 1:
+            vga256InfoRec.videoRam = 4 * 1024;
+            break;
+         case 3:
+            vga256InfoRec.videoRam = 2 * 1024;
+            break;
+         }
+      }
       else {
          switch((config1 & 0xE0) >> 5) {
          case 0:
@@ -829,12 +928,18 @@ int mclk;
       if (vga256InfoRec.dacSpeeds[2] <= 0) vga256InfoRec.dacSpeeds[2] = 135000;
       if (vga256InfoRec.dacSpeeds[3] <= 0) vga256InfoRec.dacSpeeds[3] = 135000;
    }
-   else if (s3vPriv.chip == S3_ViRGE_DXGX) {
+   else if (s3vPriv.chip == S3_ViRGE_DXGX || S3_ViRGE_GX2_SERIES(s3vPriv.chip)) {
       if (vga256InfoRec.dacSpeeds[0] <= 0) vga256InfoRec.dacSpeeds[0] = 170000;
       if (vga256InfoRec.dacSpeeds[1] <= 0) vga256InfoRec.dacSpeeds[1] = 170000;
       if (vga256InfoRec.dacSpeeds[2] <= 0) vga256InfoRec.dacSpeeds[2] = 135000;
       if (vga256InfoRec.dacSpeeds[3] <= 0) vga256InfoRec.dacSpeeds[3] = 135000;
    }
+   else if (S3_ViRGE_MX_SERIES(s3vPriv.chip)) {
+      if (vga256InfoRec.dacSpeeds[0] <= 0) vga256InfoRec.dacSpeeds[0] = 135000;
+      if (vga256InfoRec.dacSpeeds[1] <= 0) vga256InfoRec.dacSpeeds[1] = 135000;
+      if (vga256InfoRec.dacSpeeds[2] <= 0) vga256InfoRec.dacSpeeds[2] = 100000;
+      if (vga256InfoRec.dacSpeeds[3] <= 0) vga256InfoRec.dacSpeeds[3] = 100000;
+   }
    else {
       if (vga256InfoRec.dacSpeeds[0] <= 0) vga256InfoRec.dacSpeeds[0] = 135000;
       if (vga256InfoRec.dacSpeeds[1] <= 0) vga256InfoRec.dacSpeeds[1] =  95000;
@@ -898,9 +1003,44 @@ int mclk;
       }
    else s3vPriv.MCLK = 0;
 
+
+   if (S3_ViRGE_MX_SERIES(s3vPriv.chip) && xf86Verbose) {
+       int lcdclk, h_lcd, v_lcd;
+       if (OFLG_ISSET(XCONFIG_LCDCLOCK, &vga256InfoRec.xconfigFlag)) {
+	  lcdclk = vga256InfoRec.LCDClk;
+       } else {
+	  int n1, n2, sr12, sr13, sr29;
+          outb(0x3c4, 0x12);
+          sr12 = inb(0x3c5);
+          outb(0x3c4, 0x13);
+          sr13 = inb(0x3c5) & 0x7f;
+          outb(0x3c4, 0x29);
+          sr29 = inb(0x3c5);
+    	  n1 = sr12 & 0x1f;
+    	  n2 = ((sr12>>6) & 0x03) | ((sr29 & 0x01) << 2);
+          lcdclk = ((2 * 1431818 * (sr13+2)) / (n1+2) / (1 << n2) + 50) / 100;
+       }
+       outb(0x3c4, 0x61);
+       h_lcd = inb(0x3c5);
+       outb(0x3c4, 0x66);
+       h_lcd |= ((inb(0x3c5) & 0x02) << 7);
+       h_lcd = (h_lcd+1) * 8;
+       outb(0x3c4, 0x69);
+       v_lcd = inb(0x3c5);
+       outb(0x3c4, 0x6e);
+       v_lcd |= ((inb(0x3c5) & 0x70) << 4);
+       v_lcd++;
+       ErrorF("%s %s: LCD size %dx%d, clock %1.3f MHz\n"
+	      , OFLG_ISSET(XCONFIG_LCDCLOCK, &vga256InfoRec.xconfigFlag) ? XCONFIG_GIVEN : XCONFIG_PROBED
+	      , vga256InfoRec.name
+	      , h_lcd, v_lcd
+	      , lcdclk / 1000.0);
+   }
+
    /* Set scale factors for mode timings */
 
-   if (s3vPriv.chip == S3_ViRGE_VX){
+   if (s3vPriv.chip == S3_ViRGE_VX || S3_ViRGE_GX2_SERIES(s3vPriv.chip) || 
+       S3_ViRGE_MX_SERIES(s3vPriv.chip)){
       s3vPriv.HorizScaleFactor = 1;
       }
    else if (vgaBitsPerPixel == 8){
@@ -928,6 +1068,51 @@ int mclk;
       else s3vPriv.NeedSTREAMS = FALSE;
    s3vPriv.STREAMSRunning = FALSE;
 
+
+   pEnd = pMode = vga256InfoRec.modes;
+   do {
+      /* Setup the Mode.Private if required */
+      if (!pMode->PrivSize || !pMode->Private) {
+	 pMode->PrivSize = S3_MODEPRIV_SIZE;
+	 pMode->Private = (INT32 *)xcalloc(sizeof(INT32), S3_MODEPRIV_SIZE);
+	 pMode->Private[0] = 0;
+      }
+      
+      /* Set default for invert_vclk */
+      if (!(pMode->Private[0] & (1 << S3_INVERT_VCLK))) {
+	 pMode->Private[S3_INVERT_VCLK] = 0;
+	 pMode->Private[0] |= 1 << S3_INVERT_VCLK;
+      }
+      
+      /* Set default for blank_delay */
+      if (!(pMode->Private[0] & (1 << S3_BLANK_DELAY))) {
+	 pMode->Private[0] |= (1 << S3_BLANK_DELAY);
+	 if(s3vPriv.chip == S3_ViRGE_VX)
+	    /* these values need to be changed once CR67_1 is set
+	       for gamma correction (see S3V server) ! */
+	    if (vgaBitsPerPixel == 8)
+	       pMode->Private[S3_BLANK_DELAY] = 0x00;
+	    else if (vgaBitsPerPixel == 16)
+	       pMode->Private[S3_BLANK_DELAY] = 0x00;
+	    else
+	       pMode->Private[S3_BLANK_DELAY] = 0x51;
+	 else
+	    if (vgaBitsPerPixel == 8)
+	       pMode->Private[S3_BLANK_DELAY] = 0x00;
+	    else if (vgaBitsPerPixel == 16)
+	       pMode->Private[S3_BLANK_DELAY] = 0x02;
+	    else
+	       pMode->Private[S3_BLANK_DELAY] = 0x04;
+      }
+      
+      /* Set default for early_sc */
+      if (!(pMode->Private[0] & (1 << S3_EARLY_SC))) {
+	 pMode->Private[0] |= 1 << S3_EARLY_SC;
+	 pMode->Private[S3_EARLY_SC] = 0;
+      }
+      pMode = pMode->next;
+   } while (pMode != pEnd);
+
    /* And finally set various possible option flags */
 
    vga256InfoRec.bankedMono = FALSE;
@@ -946,9 +1131,12 @@ int mclk;
    OFLG_SET(OPTION_FIFO_AGGRESSIVE, &S3V.ChipOptionFlags);
    OFLG_SET(OPTION_PCI_RETRY, &S3V.ChipOptionFlags);
    OFLG_SET(OPTION_NOACCEL, &S3V.ChipOptionFlags);
+   OFLG_SET(OPTION_SW_CURSOR, &S3V.ChipOptionFlags);
    OFLG_SET(OPTION_HW_CURSOR, &S3V.ChipOptionFlags);
    OFLG_SET(OPTION_EARLY_RAS_PRECHARGE, &S3V.ChipOptionFlags);
    OFLG_SET(OPTION_LATE_RAS_PRECHARGE, &S3V.ChipOptionFlags);
+   if (S3_ViRGE_MX_SERIES(s3vPriv.chip))
+     OFLG_SET(OPTION_LCD_CENTER, &S3V.ChipOptionFlags);
 
    s3vPriv.NoPCIRetry = 1;
    S3V.ChipLinearBase = vga256InfoRec.MemBase;
@@ -993,6 +1181,18 @@ int flag;
                   vgaBitsPerPixel);
           return MODE_BAD;
         }
+	 
+	 /* For ViRGE and DX, others ?  This limit is imposed by */
+	 /* the 12 bit Stream stride register.                   */
+   if((vgaBitsPerPixel == 32) && (mode->HDisplay > 1023) &&
+	((s3vPriv.chip == S3_ViRGE) || (s3vPriv.chip == S3_ViRGE_DXGX))
+	) {
+      if(verbose)
+         ErrorF("%s %s: %s: Mode '%s' discarded. Width (%d) > 1023 at 32 bpp.\n",
+	          XCONFIG_PROBED, vga256InfoRec.name, vga256InfoRec.chipset,
+		  mode->name, mode->HDisplay);
+         return MODE_BAD;
+	}
 
    /* Now make sure we have enough vidram to support mode */
    mem = ((vga256InfoRec.displayWidth > mode->HDisplay) ? 
@@ -1024,9 +1224,12 @@ int x, y;
 int Base, hwidth;
 unsigned char tmp;
 
-   if(s3vPriv.STREAMSRunning == FALSE) {
+   if(s3vPriv.STREAMSRunning == FALSE ||
+      S3_ViRGE_GX2_SERIES(s3vPriv.chip) || S3_ViRGE_MX_SERIES(s3vPriv.chip)) {
       Base = ((y * vga256InfoRec.displayWidth + x)
 		* (vgaBitsPerPixel / 8)) >> 2;
+      if (vgaBitsPerPixel == 24) 
+	Base = Base+2 - (Base+2) % 3;
 
       /* Now program the start address registers */
       outw(vgaCRIndex, (Base & 0x00FF00) | 0x0C);
@@ -1044,6 +1247,9 @@ unsigned char tmp;
 	((y * vga256InfoRec.displayWidth + (x & ~3)) * vgaBitsPerPixel / 8);
       }
 
+   if (!OFLG_ISSET(OPTION_SW_CURSOR, &vga256InfoRec.options))
+     S3VRepositionCursor(NULL);
+
 #ifdef XFreeXDGA
    if (vga256InfoRec.directMode & XF86DGADirectGraphics) {
       /* Wait until vertical retrace is in progress. */
@@ -1131,8 +1337,12 @@ int i, j;
    new->SR15 = 0x03 | 0x80; 
    new->SR18 = 0x00;
    new->CR43 = 0x00;
-   new->CR65 = 0x20;
+   new->CR45 = 0x00;
+   new->CR65 = 0x00;
    new->CR54 = 0x00;
+
+   outb(vgaCRIndex, 0x40);
+   new->CR40 = inb(vgaCRReg) & ~0x01;
    
    /* Memory controller registers. Optimize for better graphics engine 
     * performance. These settings are adjusted/overridden below for other bpp/
@@ -1191,6 +1401,75 @@ int i, j;
 	   220000, 440000, &new->SR13, &new->SR12);
 
       }
+   else if (S3_ViRGE_GX2_SERIES(s3vPriv.chip) || S3_ViRGE_MX_SERIES(s3vPriv.chip)) {
+       if (vgaBitsPerPixel == 8)
+	  new->CR67 = 0x00;
+       else if (vgaBitsPerPixel == 16) {
+	  if (vga256InfoRec.weight.green == 5)
+	     new->CR67 = 0x30;                  /* 15bpp */
+	  else
+	     new->CR67 = 0x50;                  /* 16bpp */
+          }
+       else if ((vgaBitsPerPixel == 24) /* || (vgaBitsPerPixel == 32) */ ) {
+          new->CR67 = 0x74;              /* 24bpp, STREAMS */
+          S3VInitSTREAMS(new->STREAMS, mode);
+          new->MMPR0 = 0xc098;            /* Adjust FIFO slots */
+          }
+       else if (vgaBitsPerPixel == 32) {
+          new->CR67 = 0xd0;              /* 32bpp */
+          /* new->MMPR0 = 0xc098;            /* Adjust FIFO slots */
+          }
+       {
+         unsigned char ndiv;
+	 if (S3_ViRGE_MX_SERIES(s3vPriv.chip)) {
+	   int lcd_freq;
+	   outb(0x3c4, 0x08);  /* unlock extended SEQ regs */
+	   outb(0x3c5, 0x06);
+	   outb(0x3c4, 0x31);
+	   if (inb(0x3c5) & 0x10) { /* LCD on */
+	     if (!vga256InfoRec.LCDClk) {  /* entered only once for first mode */
+	       int h_lcd, v_lcd;
+	       outb(0x3c4, 0x61);
+	       h_lcd = inb(0x3c5);
+	       outb(0x3c4, 0x66);
+	       h_lcd |= ((inb(0x3c5) & 0x02) << 7);
+	       h_lcd = (h_lcd+1) * 8;
+	       outb(0x3c4, 0x69);
+	       v_lcd = inb(0x3c5);
+	       outb(0x3c4, 0x6e);
+	       v_lcd |= ((inb(0x3c5) & 0x70) << 4);
+	       v_lcd++;
+	       
+	       /* check if first mode has physical LCD resolution */
+	       if (vga256InfoRec.modes->HDisplay == h_lcd && vga256InfoRec.modes->VDisplay == v_lcd)
+		 vga256InfoRec.LCDClk = vga256InfoRec.clock[vga256InfoRec.modes->Clock];
+	       else {
+		 int n1, n2, sr12, sr13, sr29;
+		 outb(0x3c4, 0x12);
+		 sr12 = inb(0x3c5);
+		 outb(0x3c4, 0x13);
+		 sr13 = inb(0x3c5) & 0x7f;
+		 outb(0x3c4, 0x29);
+		 sr29 = inb(0x3c5);
+		 n1 = sr12 & 0x1f;
+		 n2 = ((sr12>>6) & 0x03) | ((sr29 & 0x01) << 2);
+		 vga256InfoRec.LCDClk = ((2 * 1431818 * (sr13+2)) / (n1+2) / (1 << n2) + 50) / 100;
+	       }
+	     }
+	     commonCalcClock(vga256InfoRec.LCDClk/2, 1, 1, 31, 0, 4,
+			     170000, 340000, &new->SR13, &ndiv);
+	   }
+	   else
+	     commonCalcClock(dclk/2, 1, 1, 31, 0, 4,
+			     170000, 340000, &new->SR13, &ndiv);
+	 }
+	 else  /* S3_ViRGE_GX2 */
+	   commonCalcClock(dclk, 1, 1, 31, 0, 4,
+			   170000, 340000, &new->SR13, &ndiv);
+         new->SR29 = ndiv >> 7;
+         new->SR12 = (ndiv & 0x1f) | ((ndiv & 0x60) << 1);
+       }
+   }
    else {           /* Is this correct for DX/GX as well? */
       if (vgaBitsPerPixel == 8) {
          if(dclk > 80000) {                     /* We need pixmux */
@@ -1283,7 +1562,6 @@ int i, j;
    width = (vga256InfoRec.displayWidth * (vgaBitsPerPixel / 8))>> 3;
    new->std.CRTC[19] = 0xFF & width;
    new->CR51 = (0x300 & width) >> 4; /* Extension bits */
-   new->CR65 = 0x20;
    
    /* And finally, select clock source 2 for programmable PLL */
    new->std.MiscOutReg |= 0x0c;      
@@ -1292,16 +1570,18 @@ int i, j;
    new->CR33 = 0x20;
    if (s3vPriv.chip == S3_ViRGE_DXGX) {
       new->CR86 = 0x80;  /* disable DAC power saving to avoid bright left edge */
+   }
+   if (s3vPriv.chip == S3_ViRGE_DXGX || S3_ViRGE_GX2_SERIES(s3vPriv.chip)) {
       new->CR90 = 0x00;  /* disable the stream display fetch length control */
    }
 	 
 
    /* Now we handle various XConfig memory options and others */
 
+   outb(vgaCRIndex, 0x36);
+   new->CR36 = inb(vgaCRReg);
    /* option "slow_edodram" sets EDO to 2 cycle mode on ViRGE */
    if (s3vPriv.chip == S3_ViRGE) {
-      outb(vgaCRIndex, 0x36);
-      new->CR36 = inb(vgaCRReg);
       if(OFLG_ISSET(OPTION_SLOW_EDODRAM, &vga256InfoRec.options)) 
          new->CR36 = (new->CR36 & 0xf3) | 0x08;
       else  
@@ -1310,20 +1590,60 @@ int i, j;
    
    /* Option "fpm_vram" for ViRGE_VX sets memory in fast page mode */
    if (s3vPriv.chip == S3_ViRGE_VX) {
-      outb(vgaCRIndex, 0x36);
-      new->CR36 = inb(vgaCRReg);
       if(OFLG_ISSET(OPTION_FPM_VRAM, &vga256InfoRec.options)) 
-         new->CR36 = new->CR36 | 0x0c;
+         new->CR36 |=  0x0c;
       else 
          new->CR36 &= ~0x0c;
       }
 
+   if (mode->Private) {
+      if (mode->Private[0] & (1 << S3_INVERT_VCLK)) {
+	 if (mode->Private[S3_INVERT_VCLK])
+	    new->CR67 |= 1;
+	 else
+	    new->CR67 &= ~1;
+      }
+      if (mode->Private[0] & (1 << S3_BLANK_DELAY)) {
+	 if (s3vPriv.chip == S3_ViRGE_VX)
+	    new->CR6D = mode->Private[S3_BLANK_DELAY];
+	 else {
+	    new->CR65 = (new->CR65 & ~0x38) 
+	       | (mode->Private[S3_BLANK_DELAY] & 0x07) << 3;
+	    outb(vgaCRIndex, 0x6d);
+	    new->CR6D = inb(vgaCRReg);
+	 }
+      }
+      if (mode->Private[0] & (1 << S3_EARLY_SC)) {
+	 if (mode->Private[S3_EARLY_SC])
+	    new->CR65 |= 2;
+	 else
+	    new->CR65 &= ~2;
+      }
+   }
+   else {
+      outb(vgaCRIndex, 0x6d);
+      new->CR6D = inb(vgaCRReg);
+   }
+
    outb(vgaCRIndex, 0x68);
    new->CR68 = inb(vgaCRReg);
    new->CR69 = 0;
 
+   if (S3_ViRGE_MX_SERIES(s3vPriv.chip) && 
+       OFLG_ISSET(OPTION_LCD_CENTER, &vga256InfoRec.options)) {
+     new->SR54 = 0x10 ;
+     new->SR55 = 0x80 ;
+     new->SR56 = 0x10 ;
+     new->SR57 = 0x80 ;
+   } else {
+     new->SR54 = 0x1f ;
+     new->SR55 = 0x9f ;
+     new->SR56 = 0x1f ;
+     new->SR57 = 0xff ;
+   }
 
-  return TRUE;
+
+   return TRUE;
 }
 
 /* This function inits the frame buffer. Right now, it is is rather limited 
@@ -1350,7 +1670,7 @@ S3VFbInit()
          ErrorF("%s %s: \"pci_retry\" option requires \"pci_burst\".\n",
               XCONFIG_GIVEN, vga256InfoRec.name);
          }
-   if (OFLG_ISSET(OPTION_HW_CURSOR, &vga256InfoRec.options)) {
+   if (!OFLG_ISSET(OPTION_SW_CURSOR, &vga256InfoRec.options)) {
       vgaHWCursor.Initialized = TRUE;
       vgaHWCursor.Init = S3VCursorInit;
       vgaHWCursor.Restore = S3VRestoreCursor;
@@ -1487,28 +1807,28 @@ int * streams;
 
    ((mmtr)s3vMmioMem)->streams_regs.regs.prim_stream_cntl = 
          streams[0] & 0x77000000;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.col_chroma_key_cntl = 0x00;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.second_stream_cntl = 0x03000000;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.chroma_key_upper_bound = 0x00;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.second_stream_stretch = 0x00;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.blend_cntl = 0x01000000;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.prim_fbaddr0 = 0x00;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.prim_fbaddr1 = 0x00;
+   ((mmtr)s3vMmioMem)->streams_regs.regs.col_chroma_key_cntl = streams[1];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.second_stream_cntl = streams[2];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.chroma_key_upper_bound = streams[3];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.second_stream_stretch = streams[4];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.blend_cntl = streams[5];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.prim_fbaddr0 = streams[6];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.prim_fbaddr1 = streams[7];
    ((mmtr)s3vMmioMem)->streams_regs.regs.prim_stream_stride = 
          streams[8] & 0x0fff;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.double_buffer = 0x00;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.second_fbaddr0 = 0x00;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.second_fbaddr1 = 0x00;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.second_stream_stride = 0x01;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.opaq_overlay_cntl = 0x40000000;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.k1 = 0x00;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.k2 = 0x00;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.dda_vert = 0x00;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.prim_start_coord = 0x00010001;
+   ((mmtr)s3vMmioMem)->streams_regs.regs.double_buffer = streams[9];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.second_fbaddr0 = streams[10];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.second_fbaddr1 = streams[11];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.second_stream_stride = streams[12];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.opaq_overlay_cntl = streams[13];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.k1 = streams[14];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.k2 = streams[15];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.dda_vert = streams[16];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.prim_start_coord = streams[18];
    ((mmtr)s3vMmioMem)->streams_regs.regs.prim_window_size = 
          streams[19] & 0x07ff07ff;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.second_start_coord = 0x07ff07ff;
-   ((mmtr)s3vMmioMem)->streams_regs.regs.second_window_size = 0x00010001;
+   ((mmtr)s3vMmioMem)->streams_regs.regs.second_start_coord = streams[20];
+   ((mmtr)s3vMmioMem)->streams_regs.regs.second_window_size = streams[21];
 
 
 }
