@@ -1,11 +1,11 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/xf86Version.h,v 3.236.2.26 1997/06/02 01:46:53 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/xf86Version.h,v 3.236.2.51 1998/07/15 11:52:33 hohndel Exp $ */
 
-#define XF86_VERSION " 3.3 "
+#define XF86_VERSION " 3.3.2.3 "
 
 /* The finer points in versions... */
 #define XF86_VERSION_MAJOR	3
 #define XF86_VERSION_MINOR	3
-#define XF86_VERSION_SUBMINOR	0
+#define XF86_VERSION_SUBMINOR	2
 #define XF86_VERSION_BETA	0	/* 0="", 1="A", 2="B", etc... */
 #define XF86_VERSION_ALPHA	0	/* 0="", 1="a", 2="b", etc... */
 
@@ -18,6 +18,6 @@
 			XF86_VERSION_BETA,			\
 			XF86_VERSION_ALPHA)
 
-#define XF86_DATE	"Jun  2 1997"
+#define XF86_DATE	"July 15 1998"
 
 /* $XConsortium: xf86Version.h /main/78 1996/10/28 05:42:10 kaleb $ */
