@@ -46,8 +46,8 @@ SOFTWARE.
 
 ******************************************************************/
 
-/* $XConsortium: WaitFor.c /main/52 1995/11/30 17:21:05 dpw $ */
-/* $XFree86: xc/programs/Xserver/os/WaitFor.c,v 3.7 1996/02/09 08:22:26 dawes Exp $ */
+/* $XConsortium: WaitFor.c /main/55 1996/12/02 10:22:24 lehors $ */
+/* $XFree86: xc/programs/Xserver/os/WaitFor.c,v 3.11.2.3 1998/01/31 14:23:33 hohndel Exp $ */
 
 /*****************************************************************
  * OS Dependent input routines:
@@ -57,6 +57,9 @@ SOFTWARE.
  *
  *****************************************************************/
 
+#ifdef WIN32
+#include <X11/Xwinsock.h>
+#endif
 #include "Xos.h"			/* for strings, fcntl, time */
 
 #include <errno.h>
@@ -80,6 +83,11 @@ extern int errno;
 #include "dixstruct.h"
 #include "opaque.h"
 
+#ifdef DPMSExtension
+#include "dpms.h"
+extern void DPMSSet();
+#endif
+
 extern fd_set AllSockets;
 extern fd_set AllClients;
 extern fd_set LastSelectMask;
@@ -142,6 +150,9 @@ WaitForSomething(pClientsReady)
     int i;
     struct timeval waittime, *wt;
     INT32 timeout;
+#ifdef DPMSExtension
+    INT32 standbyTimeout, suspendTimeout, offTimeout;
+#endif
     fd_set clientsReadable;
     fd_set clientsWritable;
     int curclient;
@@ -165,7 +176,11 @@ WaitForSomething(pClientsReady)
 	    XFD_COPYSET (&ClientsWithInput, &clientsReadable);
 	    break;
 	}
-	if (ScreenSaverTime || timers)
+#ifdef DPMSExtension
+	if (ScreenSaverTime > 0 || DPMSEnabled || timers)
+#else
+	if (ScreenSaverTime > 0 || timers)
+#endif
 	    now = GetTimeInMillis();
 	wt = NULL;
 	if (timers)
@@ -181,20 +196,51 @@ WaitForSomething(pClientsReady)
 		wt = &waittime;
 	    }
 	}
-	if (ScreenSaverTime)
+#ifdef DPMSExtension
+	if (ScreenSaverTime > 0 ||
+	    (DPMSEnabled &&
+	     (DPMSStandbyTime > 0 || DPMSSuspendTime > 0 || DPMSOffTime > 0)))
+#else
+	if (ScreenSaverTime > 0)
+#endif
 	{
+#ifdef DPMSExtension
+
+	    if (ScreenSaverTime > 0)
+		timeout = (ScreenSaverTime -
+			   (now - lastDeviceEventTime.milliseconds));
+	    if (DPMSStandbyTime > 0)
+		standbyTimeout = (DPMSStandbyTime -
+				  (now - lastDeviceEventTime.milliseconds));
+	    if (DPMSSuspendTime > 0)
+		suspendTimeout = (DPMSSuspendTime -
+				  (now - lastDeviceEventTime.milliseconds));
+	    if (DPMSOffTime > 0)
+		offTimeout = (DPMSOffTime -
+			      (now - lastDeviceEventTime.milliseconds));
+#else
 	    timeout = (ScreenSaverTime -
 		       (now - lastDeviceEventTime.milliseconds));
+#endif /* DPMSExtension */
+#ifdef DPMSExtension
+	    if (timeout <= 0 && ScreenSaverTime > 0)
+#else
 	    if (timeout <= 0) /* may be forced by AutoResetServer() */
+#endif /* DPMSExtension */
 	    {
 		INT32 timeSinceSave;
 
 		timeSinceSave = -timeout;
-		if ((timeSinceSave >= timeTilFrob) && (timeTilFrob >= 0))
+		if (timeSinceSave >= timeTilFrob && timeTilFrob >= 0)
 		{
 		    ResetOsBuffers(); /* not ideal, but better than nothing */
 		    SaveScreens(SCREEN_SAVER_ON, ScreenSaverActive);
+#ifdef DPMSExtension
+		    if (ScreenSaverInterval > 0 &&
+			DPMSPowerLevel == DPMSModeOn)
+#else
 		    if (ScreenSaverInterval)
+#endif /* DPMSExtension */
 			/* round up to the next ScreenSaverInterval */
 			timeTilFrob = ScreenSaverInterval *
 				((timeSinceSave + ScreenSaverInterval) /
@@ -206,17 +252,66 @@ WaitForSomething(pClientsReady)
 	    }
 	    else
 	    {
-		if (timeout > ScreenSaverTime)
+		if (ScreenSaverTime > 0 && timeout > ScreenSaverTime)
 		    timeout = ScreenSaverTime;
 		timeTilFrob = 0;
 	    }
-	    if (timeTilFrob >= 0 && (!wt || timeout < (timers->expires - now)))
+#ifdef DPMSExtension
+	    if (DPMSEnabled)
+	    {
+		if (standbyTimeout > 0 
+		    && (timeout <= 0 || timeout > standbyTimeout))
+		    timeout = standbyTimeout;
+		if (suspendTimeout > 0 
+		    && (timeout <= 0 || timeout > suspendTimeout))
+		    timeout = suspendTimeout;
+		if (offTimeout > 0 
+		    && (timeout <= 0 || timeout > offTimeout))
+		    timeout = offTimeout;
+	    }
+#endif
+	    if (timeout > 0 && (!wt || timeout < (timers->expires - now)))
 	    {
 		waittime.tv_sec = timeout / MILLI_PER_SECOND;
 		waittime.tv_usec = (timeout % MILLI_PER_SECOND) *
 					(1000000 / MILLI_PER_SECOND);
 		wt = &waittime;
 	    }
+#ifdef DPMSExtension
+	    /* don't bother unless it's switched on */
+	    if (DPMSEnabled)
+	    {
+		/*
+		 * If this mode's enabled, and if the time's come
+		 * and if we're still at a lesser mode, do it now.
+		 */
+		if (DPMSStandbyTime > 0) {
+		    if (standbyTimeout <= 0) {
+			if (DPMSPowerLevel < DPMSModeStandby) {
+			    DPMSSet(DPMSModeStandby);
+			}
+		    }
+		}
+		/*
+		 * and ditto.  Note that since these modes can have the
+		 * same timeouts, they can happen at the same time.
+		 */
+		if (DPMSSuspendTime > 0) {
+		    if (suspendTimeout <= 0) {
+			if (DPMSPowerLevel < DPMSModeSuspend) {
+			    DPMSSet(DPMSModeSuspend);
+			}
+		    }
+		}
+		if (DPMSOffTime > 0) {
+		    if (offTimeout <= 0) {
+			if (DPMSPowerLevel < DPMSModeOff) {
+			    DPMSSet(DPMSModeOff);
+			}
+		    }
+		}
+           }
+#endif
 	}
 	XFD_COPYSET(&AllSockets, &LastSelectMask);
 	BlockHandler((pointer)&wt, (pointer)&LastSelectMask);
@@ -259,9 +354,16 @@ WaitForSomething(pClientsReady)
 		    if (! XFD_ANYSET (&AllClients))
 			return 0;
 		}
+		else if (selecterr == EINVAL)
+		{
+		    FatalError("WaitForSomething(): select: errno=%d\n",
+			selecterr);
+		}
 		else if (selecterr != EINTR)
+		{
 		    ErrorF("WaitForSomething(): select: errno=%d\n",
 			selecterr);
+		}
 	    if (timers)
 	    {
 		now = GetTimeInMillis();
@@ -273,6 +375,9 @@ WaitForSomething(pClientsReady)
 	}
 	else
 	{
+#ifdef WIN32
+	    fd_set tmp_set;
+#endif
 	    if (AnyClientsWriteBlocked && XFD_ANYSET (&clientsWritable))
 	    {
 		NewOutputPending = TRUE;
@@ -284,9 +389,18 @@ WaitForSomething(pClientsReady)
 
 	    XFD_ANDSET(&devicesReadable, &LastSelectMask, &EnabledDevices);
 	    XFD_ANDSET(&clientsReadable, &LastSelectMask, &AllClients); 
+#ifndef WIN32
 	    if (LastSelectMask.fds_bits[0] & WellKnownConnections.fds_bits[0]) 
+#else
+	    XFD_ANDSET(&tmp_set, &LastSelectMask, &WellKnownConnections);
+	    if (XFD_ANYSET(&tmp_set))
+#endif
 		QueueWorkProc(EstablishNewConnections, NULL,
 			      (pointer)&LastSelectMask);
+#ifdef DPMSExtension
+	    if (XFD_ANYSET (&devicesReadable) && (DPMSPowerLevel != DPMSModeOn))
+		DPMSSet(DPMSModeOn);
+#endif
 	    if (XFD_ANYSET (&devicesReadable) || XFD_ANYSET (&clientsReadable))
 		break;
 	}
@@ -295,6 +409,7 @@ WaitForSomething(pClientsReady)
     nready = 0;
     if (XFD_ANYSET (&clientsReadable))
     {
+#ifndef WIN32
 	for (i=0; i<howmany(XFD_SETSIZE, NFDBITS); i++)
 	{
 	    int highest_priority;
@@ -305,6 +420,17 @@ WaitForSomething(pClientsReady)
 
 		curclient = ffs (clientsReadable.fds_bits[i]) - 1;
 		client_index = ConnectionTranslation[curclient + (i << 5)];
+#else
+	int highest_priority;
+	fd_set savedClientsReadable;
+	XFD_COPYSET(&clientsReadable, &savedClientsReadable);
+	for (i = 0; i < XFD_SETCOUNT(&savedClientsReadable); i++)
+	{
+	    int client_priority, client_index;
+
+	    curclient = XFD_FD(&savedClientsReadable, i);
+	    client_index = ConnectionTranslation[curclient];
+#endif
 #ifdef XSYNC
 		/*  We implement "strict" priorities.
 		 *  Only the highest priority client is returned to
@@ -336,9 +462,13 @@ WaitForSomething(pClientsReady)
 		{
 		    pClientsReady[nready++] = client_index;
 		}
-		clientsReadable.fds_bits[i] &= ~(((FdMask)1) << curclient);
+#ifndef WIN32
+		clientsReadable.fds_bits[i] &= ~(((fd_mask)1) << curclient);
 	    }
-	}	
+#else
+	    FD_CLR(curclient, &clientsReadable);
+#endif
+	}
     }
     return nready;
 }
@@ -607,8 +737,9 @@ TimerForce(timer)
     return FALSE;
 }
 
+
 void
-TimerFree(timer)
+TimerCancel(timer)
     register OsTimerPtr timer;
 {
     register OsTimerPtr *prev;
@@ -623,6 +754,15 @@ TimerFree(timer)
 	    break;
 	}
     }
+}
+
+void
+TimerFree(timer)
+    register OsTimerPtr timer;
+{
+    if (!timer)
+	return;
+    TimerCancel(timer);
     xfree(timer);
 }
 
