@@ -1,6 +1,6 @@
 /*
- * $XConsortium: Tekproc.c /main/118 1996/01/14 16:52:29 kaleb $
- * $XFree86: xc/programs/xterm/Tekproc.c,v 3.12 1996/08/13 11:36:43 dawes Exp $
+ * $XConsortium: Tekproc.c /main/120 1996/11/29 10:33:20 swick $
+ * $XFree86: xc/programs/xterm/Tekproc.c,v 3.13.2.4 1998/04/29 11:18:06 dawes Exp $
  *
  * Warning, there be crufty dragons here.
  */
@@ -56,6 +56,10 @@ in this Software without prior written authorization from the X Consortium.
 
 /* Tekproc.c */
 
+#ifdef HAVE_CONFIG_H
+#include <xtermcfg.h>
+#endif
+
 #include "ptyx.h"
 #include <X11/Xos.h>
 #include <X11/Xatom.h>
@@ -70,6 +74,9 @@ in this Software without prior written authorization from the X Consortium.
 #include <errno.h>
 #include <setjmp.h>
 #include <signal.h>
+
+#include "xterm.h"
+
 #include "Tekparse.h"
 #include "data.h"
 #include "error.h"
@@ -117,10 +124,6 @@ extern void exit();
 extern long time();		/* included in <time.h> by Xos.h */
 #endif
 
-#include "xterm.h"
-
-#define TekColormap DefaultColormap( screen->display, \
-				    DefaultScreen(screen->display) )
 #define DefaultGCID XGContextFromGC(DefaultGC(screen->display, DefaultScreen(screen->display)))
 
 /* Tek defines */
@@ -678,7 +681,7 @@ static void Tekparse()
 				int c2, len = 0;
 				while ((c2 = Tinput()) != BEL) {
 					if (!isprint(c2 & 0x7f)
-					 || len+2 >= sizeof(buf2))
+					 || len+2 >= (int) sizeof(buf2))
 						break;
 					buf2[len++] = c2;
 				}
@@ -861,9 +864,9 @@ static void TekConfigure(w)
 /*ARGSUSED*/
 void
 TekExpose(w, event, region)
-    Widget w;
-    XEvent *event;
-    Region region;
+    Widget w GCC_UNUSED;
+    XEvent *event GCC_UNUSED;
+    Region region GCC_UNUSED;
 {
 	register TScreen *screen = &term->screen;
 
@@ -1265,9 +1268,10 @@ static unsigned char *dashes[TEKNUMLINES] = {
  */
 
 static void TekInitialize(request, new, args, num_args)
-    Widget request, new;
-    ArgList args;
-    Cardinal *num_args;
+    Widget request GCC_UNUSED;
+    Widget new GCC_UNUSED;
+    ArgList args GCC_UNUSED;
+    Cardinal *num_args GCC_UNUSED;
 {
     /* look for focus related events on the shell, because we need
      * to care about the shell's border being part of our focus.
@@ -1300,6 +1304,10 @@ static void TekRealize (gw, valuemaskp, values)
     XSizeHints sizehints;
     char Tdefault[32];
 
+#ifndef NO_ACTIVE_ICON
+    term->screen.whichTwin = &term->screen.fullTwin;
+#endif /* NO_ACTIVE_ICON */
+
     tw->core.border_pixel = term->core.border_pixel;
 
     for (i = 0; i < TEKNUMFONTS; i++) {
@@ -1716,8 +1724,10 @@ void TekSimulatePageButton (reset)
 
 
 #ifndef X_NOT_POSIX
+#ifndef linux
 #define HAS_WAITPID
 #endif
+#endif
 
 #ifdef HAS_WAITPID
 #include <sys/wait.h>
@@ -1735,14 +1745,14 @@ TekCopy()
 	int pid;
 #ifndef HAS_WAITPID
 	int waited;
-	int (*chldfunc)();
+	SIGNAL_T (*chldfunc) PROTO((int n));
 
 	chldfunc = signal(SIGCHLD, SIG_DFL);
 #endif
 
 	time(&l);
 	tp = localtime(&l);
-	sprintf(buf, "COPY%02d-%02d-%02d.%02d:%02d:%02d", tp->tm_year,
+	sprintf(buf, "COPY%d-%02d-%02d.%02d:%02d:%02d", tp->tm_year + 1900,
 	 tp->tm_mon + 1, tp->tm_mday, tp->tm_hour, tp->tm_min, tp->tm_sec);
 	if(access(buf, F_OK) >= 0) {	/* file exists */
 		if(access(buf, W_OK) < 0) {
