@@ -5,7 +5,7 @@
 #ifndef lint
 static char *rid="$XConsortium: main.c,v 1.227.1.2 95/06/29 18:13:15 kaleb Exp $";
 #endif /* lint */
-/* $XFree86: xc/programs/xterm/os2main.c,v 3.4 1996/08/20 12:33:54 dawes Exp $ */
+/* $XFree86: xc/programs/xterm/os2main.c,v 3.5.2.5 1998/04/29 10:52:01 dawes Exp $ */
 
 /***********************************************************
 
@@ -66,21 +66,23 @@ SOFTWARE.
 #include <os2.h>
 #endif
 
+#include "version.h"
 #include "ptyx.h"
-#include "data.h"
-#include "error.h"
-#include "menu.h"
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
-#ifdef I18N
-#include <X11/Xlocale.h>
-#endif
 
 #include <X11/Xos.h>
 #include <X11/cursorfont.h>
 #include <X11/Xaw/SimpleMenu.h>
+#ifdef I18N
+#include <X11/Xlocale.h>
+#endif
+
 #include <pwd.h>
 #include <ctype.h>
+#include "data.h"
+#include "error.h"
+#include "menu.h"
 
 #include <sys/termio.h>
 
@@ -106,17 +108,28 @@ char *ttyname(int fd) { return "/dev/tty"; }
 #define SIGNAL_T void
 #define SIGNAL_RETURN return
 
-SIGNAL_T Exit();
-
 #include <unistd.h>
 #include <stdlib.h>
 
 extern char *strindex ();
-extern void HandlePopupMenu();
+
+#include "xterm.h"
 
 int switchfb[] = {0, 2, 1, 3};
 
-static SIGNAL_T reapchild ();
+static char *base_name PROTO((char *name));
+static int pty_search PROTO((int *pty));
+static int remove_termcap_entry PROTO((char *buf, char *str));
+static int spawn PROTO((void));
+static void DeleteWindow PROTO_XT_ACTIONS_ARGS;
+static void Help PROTO((void));
+static void KeyboardMapping PROTO_XT_ACTIONS_ARGS;
+static void Syntax PROTO((char *badOption));
+static void Version PROTO((void));
+static void get_terminal PROTO((void));
+static void my_error_handler PROTO((String message));
+static void resize PROTO((TScreen *s, char *oldtc, char *newtc));
+static SIGNAL_T reapchild PROTO((int n));
 
 static Bool added_utmp_entry = False;
 
@@ -163,7 +176,6 @@ static struct termio d_tio;
 #define CSTART ('Q'&037)
 #endif
 
-static int parse_tty_modes ();
 /*
  * SYSV has the termio.c_cc[V] and ltchars; BSD has tchars and ltchars;
  * SVR4 has only termio.c_cc, but it includes everything from ltchars.
@@ -210,6 +222,8 @@ struct _xttymodes {
 { NULL, 0, 0, '\0' },			/* end of data */
 };
 
+static int parse_tty_modes PROTO((char *s, struct _xttymodes *modelist));
+
 static int inhibit;
 static char passedPty[2];	/* name if pty if slave */
 
@@ -239,6 +253,9 @@ static struct _resource {
     char *tty_modes;
     Boolean utmpInhibit;
     Boolean sunFunctionKeys;	/* %%% should be widget resource? */
+#if OPT_SUNPC_KBD
+    Boolean sunKeyboard;
+#endif
     Boolean wait_for_map;
     Boolean useInsertMode;
 } resource;
@@ -249,7 +266,7 @@ static struct _resource {
 
 static XtResource application_resources[] = {
     {"name", "Name", XtRString, sizeof(char *),
-	offset(xterm_name), XtRString, "xterm"},
+	offset(xterm_name), XtRString, DFT_TERMTYPE},
     {"iconGeometry", "IconGeometry", XtRString, sizeof(char *),
 	offset(icon_geometry), XtRString, (caddr_t) NULL},
     {XtNtitle, XtCTitle, XtRString, sizeof(char *),
@@ -264,6 +281,10 @@ static XtResource application_resources[] = {
 	offset(utmpInhibit), XtRString, "false"},
     {"sunFunctionKeys", "SunFunctionKeys", XtRBoolean, sizeof (Boolean),
 	offset(sunFunctionKeys), XtRString, "false"},
+#if OPT_SUNPC_KBD
+    {"sunKeyboard", "SunKeyboard", XtRBoolean, sizeof (Boolean),
+	offset(sunKeyboard), XtRString, "false"},
+#endif
     {"waitForMap", "WaitForMap", XtRBoolean, sizeof (Boolean),
         offset(wait_for_map), XtRString, "false"},
     {"useInsertMode", "UseInsertMode", XtRBoolean, sizeof (Boolean),
@@ -279,7 +300,9 @@ static char *fallback_resources[] = {
     "XTerm*mainMenu.Label:  Main Options (no app-defaults)",
     "XTerm*vtMenu.Label:  VT Options (no app-defaults)",
     "XTerm*fontMenu.Label:  VT Fonts (no app-defaults)",
+#if OPT_TEK4014
     "XTerm*tekMenu.Label:  Tek Options (no app-defaults)",
+#endif
     NULL
 };
 
@@ -292,17 +315,35 @@ static XrmOptionDescRec optionDescList[] = {
 {"+132",	"*c132",	XrmoptionNoArg,		(caddr_t) "off"},
 {"-ah",		"*alwaysHighlight", XrmoptionNoArg,	(caddr_t) "on"},
 {"+ah",		"*alwaysHighlight", XrmoptionNoArg,	(caddr_t) "off"},
+{"-aw",		"*autoWrap",	XrmoptionNoArg,		(caddr_t) "on"},
+{"+aw",		"*autoWrap",	XrmoptionNoArg,		(caddr_t) "off"},
+#ifndef NO_ACTIVE_ICON
+{"-ai",		"*activeIcon",	XrmoptionNoArg,		(caddr_t) "off"},
+{"+ai",		"*activeIcon",	XrmoptionNoArg,		(caddr_t) "on"},
+#endif /* NO_ACTIVE_ICON */
 {"-b",		"*internalBorder",XrmoptionSepArg,	(caddr_t) NULL},
+{"-bdc",	"*colorBDMode",	XrmoptionNoArg,		(caddr_t) "off"},
+{"+bdc",	"*colorBDMode",	XrmoptionNoArg,		(caddr_t) "on"},
 {"-cb",		"*cutToBeginningOfLine", XrmoptionNoArg, (caddr_t) "off"},
 {"+cb",		"*cutToBeginningOfLine", XrmoptionNoArg, (caddr_t) "on"},
 {"-cc",		"*charClass",	XrmoptionSepArg,	(caddr_t) NULL},
+{"-cm",		"*colorMode",	XrmoptionNoArg,		(caddr_t) "off"},
+{"+cm",		"*colorMode",	XrmoptionNoArg,		(caddr_t) "on"},
 {"-cn",		"*cutNewline",	XrmoptionNoArg,		(caddr_t) "off"},
 {"+cn",		"*cutNewline",	XrmoptionNoArg,		(caddr_t) "on"},
 {"-cr",		"*cursorColor",	XrmoptionSepArg,	(caddr_t) NULL},
 {"-cu",		"*curses",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+cu",		"*curses",	XrmoptionNoArg,		(caddr_t) "off"},
+{"-dc",		"*dynamicColors",XrmoptionNoArg,	(caddr_t) "off"},
+{"+dc",		"*dynamicColors",XrmoptionNoArg,	(caddr_t) "on"},
 {"-e",		NULL,		XrmoptionSkipLine,	(caddr_t) NULL},
 {"-fb",		"*boldFont",	XrmoptionSepArg,	(caddr_t) NULL},
+#ifndef NO_ACTIVE_ICON
+{"-fi",		"*iconFont",	XrmoptionSepArg,	(caddr_t) NULL},
+#endif /* NO_ACTIVE_ICON */
+#if OPT_HIGHLIGHT_COLOR
+{"-hc",		"*highlightColor", XrmoptionSepArg,	(caddr_t) NULL},
+#endif
 {"-j",		"*jumpScroll",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+j",		"*jumpScroll",	XrmoptionNoArg,		(caddr_t) "off"},
 /* parse logging options anyway for compatibility */
@@ -316,25 +357,37 @@ static XrmOptionDescRec optionDescList[] = {
 {"-mc",		"*multiClickTime", XrmoptionSepArg,	(caddr_t) NULL},
 {"-ms",		"*pointerColor",XrmoptionSepArg,	(caddr_t) NULL},
 {"-nb",		"*nMarginBell",	XrmoptionSepArg,	(caddr_t) NULL},
+{"-nul",	"*underLine",	XrmoptionNoArg,		(caddr_t) "off"},
+{"+nul",	"*underLine",	XrmoptionNoArg,		(caddr_t) "on"},
+{"-pc",		"*boldColors",	XrmoptionNoArg,		(caddr_t) "on"},
+{"+pc",		"*boldColors",	XrmoptionNoArg,		(caddr_t) "off"},
 {"-rw",		"*reverseWrap",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+rw",		"*reverseWrap",	XrmoptionNoArg,		(caddr_t) "off"},
-{"-aw",		"*autoWrap",	XrmoptionNoArg,		(caddr_t) "on"},
-{"+aw",		"*autoWrap",	XrmoptionNoArg,		(caddr_t) "off"},
 {"-s",		"*multiScroll",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+s",		"*multiScroll",	XrmoptionNoArg,		(caddr_t) "off"},
 {"-sb",		"*scrollBar",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+sb",		"*scrollBar",	XrmoptionNoArg,		(caddr_t) "off"},
+#ifdef SCROLLBAR_RIGHT
+{"-leftbar",	"*rightScrollBar", XrmoptionNoArg,	(caddr_t) "off"},
+{"-rightbar",	"*rightScrollBar", XrmoptionNoArg,	(caddr_t) "on"},
+#endif
 {"-sf",		"*sunFunctionKeys", XrmoptionNoArg,	(caddr_t) "on"},
 {"+sf",		"*sunFunctionKeys", XrmoptionNoArg,	(caddr_t) "off"},
-{"-si",		"*scrollTtyOutput",	XrmoptionNoArg,		(caddr_t) "off"},
-{"+si",		"*scrollTtyOutput",	XrmoptionNoArg,		(caddr_t) "on"},
+{"-si",		"*scrollTtyOutput", XrmoptionNoArg,	(caddr_t) "off"},
+{"+si",		"*scrollTtyOutput", XrmoptionNoArg,	(caddr_t) "on"},
 {"-sk",		"*scrollKey",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+sk",		"*scrollKey",	XrmoptionNoArg,		(caddr_t) "off"},
 {"-sl",		"*saveLines",	XrmoptionSepArg,	(caddr_t) NULL},
+#if OPT_SUNPC_KBD
+{"-sp",		"*sunKeyboard", XrmoptionNoArg,		(caddr_t) "on"},
+{"+sp",		"*sunKeyboard", XrmoptionNoArg,		(caddr_t) "off"},
+#endif
 {"-t",		"*tekStartup",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+t",		"*tekStartup",	XrmoptionNoArg,		(caddr_t) "off"},
 {"-tm",		"*ttyModes",	XrmoptionSepArg,	(caddr_t) NULL},
 {"-tn",		"*termName",	XrmoptionSepArg,	(caddr_t) NULL},
+{"-ulc",	"*colorULMode",	XrmoptionNoArg,		(caddr_t) "off"},
+{"+ulc",	"*colorULMode",	XrmoptionNoArg,		(caddr_t) "on"},
 {"-ut",		"*utmpInhibit",	XrmoptionNoArg,		(caddr_t) "on"},
 {"+ut",		"*utmpInhibit",	XrmoptionNoArg,		(caddr_t) "off"},
 {"-im",		"*useInsertMode", XrmoptionNoArg,	(caddr_t) "on"},
@@ -360,6 +413,7 @@ static struct _options {
   char *opt;
   char *desc;
 } options[] = {
+{ "-version",              "print the version number" },
 { "-help",                 "print out this message" },
 { "-display displayname",  "X server to contact" },
 { "-geometry geom",        "size (in characters) and position" },
@@ -375,13 +429,23 @@ static struct _options {
 { "-xrm resourcestring",   "additional resource specifications" },
 { "-/+132",                "turn on/off column switch inhibiting" },
 { "-/+ah",                 "turn on/off always highlight" },
+#ifndef NO_ACTIVE_ICON
+{ "-/+ai",		   "turn on/off active icon" },
+{ "-fi fontname",	   "icon font for active icon" },
+#endif /* NO_ACTIVE_ICON */
 { "-b number",             "internal border in pixels" },
+{ "-/+bdc",                "turn off/on display of bold as color"},
 { "-/+cb",                 "turn on/off cut-to-beginning-of-line inhibit" },
 { "-cc classrange",        "specify additional character classes" },
+{ "-/+cm",                 "turn off/on ANSI color mode" },
 { "-/+cn",                 "turn on/off cut newline inhibit" },
 { "-cr color",             "text cursor color" },
 { "-/+cu",                 "turn on/off curses emulation" },
+{ "-/+dc",		   "turn off/on dynamic color selection" },
 { "-fb fontname",          "bold text font" },
+#if OPT_HIGHLIGHT_COLOR
+{ "-hc",		   "selection background color" },
+#endif
 { "-/+im",		   "use insert mode for TERMCAP" },
 { "-/+j",                  "turn on/off jump scroll" },
 #ifdef ALLOWLOGGING
@@ -396,22 +460,34 @@ static struct _options {
 { "-mc milliseconds",      "multiclick time in milliseconds" },
 { "-ms color",             "pointer color" },
 { "-nb number",            "margin bell in characters from right end" },
+{ "-/+nul",                "turn on/off display of underlining" },
 { "-/+aw",                 "turn on/off auto wraparound" },
+{ "-/+pc",                 "turn on/off PC-style bold colors" },
 { "-/+rw",                 "turn on/off reverse wraparound" },
 { "-/+s",                  "turn on/off multiscroll" },
 { "-/+sb",                 "turn on/off scrollbar" },
+#ifdef SCROLLBAR_RIGHT
+{ "-rightbar",             "force scrollbar right (default left)" },
+{ "-leftbar",              "force scrollbar left" },
+#endif
 { "-/+sf",                 "turn on/off Sun Function Key escape codes" },
 { "-/+si",                 "turn on/off scroll-on-tty-output inhibit" },
 { "-/+sk",                 "turn on/off scroll-on-keypress" },
 { "-sl number",            "number of scrolled lines to save" },
+{ "-/+sp",                 "turn on/off Sun/PC Function/Keypad mapping" },
+#if OPT_TEK4014
 { "-/+t",                  "turn on/off Tek emulation window" },
+#endif
 { "-tm string",            "terminal mode keywords and characters" },
 { "-tn name",              "TERM environment variable name" },
+{ "-/+ulc",                "turn off/on display of underline as color" },
 { "-/+ut",                 "turn on/off utmp inhibit (not supported)" },
 { "-/+vb",                 "turn on/off visual bell" },
 { "-/+wf",                 "turn on/off wait for map before command exec" },
 { "-e command args ...",   "command to execute" },
+#if OPT_TEK4014
 { "%geom",                 "Tek window geometry" },
+#endif
 { "#geom",                 "icon window geometry" },
 { "-T string",             "title name for window" },
 { "-n string",             "icon name for window" },
@@ -435,14 +511,19 @@ static void closecons()
 }
 */
 static char *message[] = {
-"Fonts must be fixed width and, if both normal and bold are specified, must",
+"Fonts should be fixed width and, if both normal and bold are specified, should",
 "have the same size.  If only a normal font is specified, it will be used for",
 "both normal and bold text (by doing overstriking).  The -e option, if given,",
-"must be appear at the end of the command line, otherwise the user's default",
-"shell will be started.  Options that start with a plus sign (+) restore the",
-"default.",
+"must appear at the end of the command line, otherwise the user's default shell",
+"will be started.  Options that start with a plus sign (+) restore the default.",
 NULL};
 
+static int abbrev (char *tst, char *cmp)
+{
+	size_t len = strlen(tst);
+	return ((len >= 2) && (!strncmp(tst, cmp, len)));
+}
+
 static void Syntax (badOption)
     char *badOption;
 {
@@ -469,13 +550,19 @@ static void Syntax (badOption)
     exit (1);
 }
 
+static void Version ()
+{
+    puts (XTERM_VERSION);
+    exit (0);
+}
+
 static void Help ()
 {
     struct _options *opt;
     char **cpp;
 
-    fprintf (stderr, "usage:\n        %s [-options ...] [-e command args]\n\n",
-	     ProgramName);
+    fprintf (stderr, "%s usage:\n    %s [-options ...] [-e command args]\n\n",
+	     XTERM_VERSION, ProgramName);
     fprintf (stderr, "where options include:\n");
     for (opt = options; opt->opt; opt++) {
 	fprintf (stderr, "    %-28s %s\n", opt->opt, opt->desc);
@@ -494,62 +581,57 @@ static void Help ()
 /* ARGSUSED */
 static Boolean
 ConvertConsoleSelection(w, selection, target, type, value, length, format)
-    Widget w;
-    Atom *selection, *target, *type;
-    XtPointer *value;
-    unsigned long *length;
-    int *format;
+    Widget w GCC_UNUSED;
+    Atom *selection GCC_UNUSED, *target GCC_UNUSED, *type GCC_UNUSED;
+    XtPointer *value GCC_UNUSED;
+    unsigned long *length GCC_UNUSED;
+    int *format GCC_UNUSED;
 {
     /* we don't save console output, so can't offer it */
     return False;
 }
 
-
-extern WidgetClass xtermWidgetClass;
-
 Arg ourTopLevelShellArgs[] = {
 	{ XtNallowShellResize, (XtArgVal) TRUE },	
 	{ XtNinput, (XtArgVal) TRUE },
 };
 int number_ourTopLevelShellArgs = 2;
 	
-XtAppContext app_con;
 Widget toplevel;
 Bool waiting_for_initial_map;
 
-extern void do_hangup();
-extern void xt_error();
-
 /*
  * DeleteWindow(): Action proc to implement ICCCM delete_window.
  */
 /* ARGSUSED */
-void
+static void
 DeleteWindow(w, event, params, num_params)
     Widget w;
-    XEvent *event;
-    String *params;
-    Cardinal *num_params;
+    XEvent *event GCC_UNUSED;
+    String *params GCC_UNUSED;
+    Cardinal *num_params GCC_UNUSED;
 {
+#if OPT_TEK4014
   if (w == toplevel)
     if (term->screen.Tshow)
       hide_vt_window();
     else
-      do_hangup(w);
+      do_hangup(w, (XtPointer)0, (XtPointer)0);
   else
     if (term->screen.Vshow)
       hide_tek_window();
     else
-      do_hangup(w);
+#endif
+      do_hangup(w, (XtPointer)0, (XtPointer)0);
 }
 
 /* ARGSUSED */
-void
+static void
 KeyboardMapping(w, event, params, num_params)
-    Widget w;
+    Widget w GCC_UNUSED;
     XEvent *event;
-    String *params;
-    Cardinal *num_params;
+    String *params GCC_UNUSED;
+    Cardinal *num_params GCC_UNUSED;
 {
     switch (event->type) {
        case MappingNotify:
@@ -559,8 +641,8 @@ KeyboardMapping(w, event, params, num_params)
 }
 
 XtActionsRec actionProcs[] = {
-    "DeleteWindow", DeleteWindow,
-    "KeyboardMapping", KeyboardMapping,
+    { "DeleteWindow", DeleteWindow },
+    { "KeyboardMapping", KeyboardMapping },
 };
 
 Atom wm_delete_window;
@@ -685,17 +767,21 @@ int ptioctl(int fd, int func, void* data)
 
 char **gblenvp;
 
-main (argc, argv, envp)
-int argc;
-char **argv;
-char **envp;
+main (int argc, char** argv, char** envp)
 {
 	register TScreen *screen;
 	int mode;
-	char *base_name();
-	int xerror(), xioerror();
 	extern char **environ;
 
+	/* Do these first, since we may not be able to open the display */
+	ProgramName = argv[0];
+	if (argc > 1) {
+		if (abbrev(argv[1], "-version"))
+			Version();
+		if (abbrev(argv[1], "-help"))
+			Help();
+	}
+
 	/* XXX: for some obscure reason EMX seems to lose the value of 
 	 * the environ variable, don't understand why, so save it recently
 	 */
@@ -705,8 +791,6 @@ char **envp;
 	setlocale(LC_ALL, NULL);
 #endif
 
-	ProgramName = argv[0];
-
 /*debug	opencons();*/
 
 	ttydev = (char *) malloc (PTMS_BUFSZ);
@@ -743,9 +827,10 @@ char **envp;
 	    XtSetErrorHandler(xt_error);
 	    toplevel = XtAppInitialize (&app_con, "XTerm", 
 					optionDescList,
-					XtNumber(optionDescList), 
+					XtNumber(optionDescList),
 					&argc, argv, fallback_resources,
 					NULL, 0);
+	    XtSetErrorHandler((XtErrorHandler)0);
 
 	    XtGetApplicationResources(toplevel, (XtPointer) &resource,
 				      application_resources,
@@ -774,7 +859,10 @@ char **envp;
 
 	xterm_name = resource.xterm_name;
 	sunFunctionKeys = resource.sunFunctionKeys;
-	if (strcmp(xterm_name, "-") == 0) xterm_name = "xterm";
+#if OPT_SUNPC_KBD
+	sunKeyboard = resource.sunKeyboard;
+#endif
+	if (strcmp(xterm_name, "-") == 0) xterm_name = DFT_TERMTYPE;
 	if (resource.icon_geometry != NULL) {
 	    int scr, junk;
 	    int ix, iy;
@@ -846,9 +934,10 @@ char **envp;
 
         term = (XtermWidget) XtCreateManagedWidget(
 	    "vt100", xtermWidgetClass, toplevel, NULL, 0);
+	    /* this causes the initialize method to be called */
 
-	/* this causes the initialize method to be called */
         screen = &term->screen;
+
 	if (screen->savelines < 0) screen->savelines = 0;
 
 	term->flags = 0;
@@ -871,20 +960,22 @@ char **envp;
 
 	inhibit = 0;
 #ifdef ALLOWLOGGING
-	if (term->misc.logInhibit) 	    inhibit |= I_LOG;
+	if (term->misc.logInhibit)		inhibit |= I_LOG;
 #endif
 	if (term->misc.signalInhibit)		inhibit |= I_SIGNAL;
-	if (term->misc.tekInhibit)			inhibit |= I_TEK;
+#if OPT_TEK4014
+	if (term->misc.tekInhibit)		inhibit |= I_TEK;
+#endif
 
 	term->initflags = term->flags;
 
 	if (term->misc.appcursorDefault) {
-	    term->keyboard.flags |= CURSOR_APL;
+	    term->keyboard.flags |= MODE_DECCKM;
 	    update_appcursor();
 	}
 
 	if (term->misc.appkeypadDefault) {
-	    term->keyboard.flags |= KYPD_APL;
+	    term->keyboard.flags |= MODE_DECKPAM;
 	    update_appkeypad();
 	}
 
@@ -909,11 +1000,13 @@ char **envp;
 	    XtSetValues (toplevel, args, 2);
 	}
 
+#if OPT_TEK4014
 	if(inhibit & I_TEK)
 		screen->TekEmu = FALSE;
 
 	if(screen->TekEmu && !TekInit())
 		exit(ERROR_INIT);
+#endif
 
 #ifdef DEBUG
     {
@@ -929,7 +1022,7 @@ char **envp;
 		dup2(i,2);
 
 		/* mark this file as close on exec */
-		fcntl(i, F_SETFD, 1);
+		(void) fcntl(i, F_SETFD, 1);
 	}
     }
 #endif	/* DEBUG */
@@ -940,7 +1033,7 @@ char **envp;
 	spawn ();
 
 	/* Child process is out there, let's catch its termination */
-	signal (SIGCHLD, reapchild);
+	(void) signal (SIGCHLD, reapchild);
 
 	/* Realize procs have now been executed */
 
@@ -948,25 +1041,18 @@ char **envp;
 	    char buf[80];
 
 	    buf[0] = '\0';
-	    sprintf (buf, "%lx\n", 
-	    	     screen->TekEmu ? XtWindow (XtParent (tekWidget)) :
-				      XtWindow (XtParent (term)));
+	    sprintf (buf, "%lx\n", XtWindow (XtParent (CURRENT_EMU(screen))));
 	    write (screen->respond, buf, strlen (buf));
 	}
 
-#ifdef ALLOWLOGGING
-	if (term->misc.log_on) {
-		StartLog(screen);
-	}
-#endif
 	screen->inhibit = inhibit;
 
 	if (0 > (mode = fcntl(screen->respond, F_GETFL, 0)))
-		Error();
+		Error(1);
 	mode |= O_NDELAY;
 
 	if (fcntl(screen->respond, F_SETFL, mode))
-		Error();
+		Error(1);
 
 	FD_ZERO (&pty_mask);
 	FD_ZERO (&X_mask);
@@ -978,23 +1064,30 @@ char **envp;
 	max_plus1 = (screen->respond < ConnectionNumber(screen->display)) ? 
 		(1 + ConnectionNumber(screen->display)) : 
 		(1 + screen->respond);
-	
+
 #ifdef DEBUG
 	if (debug) printf ("debugging on\n");
 #endif	/* DEBUG */
 	XSetErrorHandler(xerror);
 	XSetIOErrorHandler(xioerror);
 
+#ifdef ALLOWLOGGING
+	if (term->misc.log_on) {
+		StartLog(screen);
+	}
+#endif
 	for( ; ; ) {
-		if(screen->TekEmu) {
+#if OPT_TEK4014
+		if(screen->TekEmu)
 			TekRun();
-		} else {
+		else
+#endif
 			VTRun();
-		}
 	}
 }
 
-char *base_name(name)
+static char *
+base_name(name)
 char *name;
 {
 	register char *cp;
@@ -1010,6 +1103,7 @@ char *name;
  * has problems, we can re-enter this function and get another one.
  */
 
+static int
 get_pty (pty)
     int *pty;
 {
@@ -1022,7 +1116,8 @@ get_pty (pty)
  * a functional interface for allocating a pty.
  * Returns 0 if found a pty, 1 if fails.
  */
-int pty_search(pty)
+static int
+pty_search(pty)
     int *pty;
 {
 	char namebuf[PTMS_BUFSZ];
@@ -1044,6 +1139,7 @@ int pty_search(pty)
 	return 1;
 }
 
+static void
 get_terminal ()
 /* 
  * sets up X and initializes the terminal structure except for term.buf.fildes.
@@ -1069,6 +1165,7 @@ get_terminal ()
  * All of these are supported by xterm.
  */
 
+#if OPT_TEK4014
 static char *tekterm[] = {
 	"tek4014",
 	"tek4015",		/* 4014 with APL character set support */
@@ -1078,6 +1175,7 @@ static char *tekterm[] = {
 	"dumb",
 	0
 };
+#endif
 
 /* The VT102 is a VT100 with the Advanced Video Option included standard.
  * It also adds Escape sequences for insert/delete character/line.
@@ -1090,6 +1188,7 @@ static char *vtterm[] = {
 #ifdef USE_X11TERM
 	"x11term",		/* for people who want special term name */
 #endif
+	DFT_TERMTYPE,		/* for people who want special term name */
 	"xterm",		/* the prefered name, should be fastest */
 	"vt102",
 	"vt100",
@@ -1099,8 +1198,8 @@ static char *vtterm[] = {
 };
 
 /* ARGSUSED */
-SIGNAL_T hungtty(i)
-	int i;
+static SIGNAL_T hungtty(i)
+	int i GCC_UNUSED;
 {
 	longjmp(env, 1);
 	SIGNAL_RETURN;
@@ -1146,6 +1245,7 @@ SIGNAL_T killit(int sig)
 	SIGNAL_RETURN;
 }
 
+static int
 spawn ()
 /* 
  *  Inits pty and tty and forks a login process.
@@ -1153,8 +1253,6 @@ spawn ()
  *  If slave, the pty named in passedPty is already open for use
  */
 {
-	extern char *SysErrorMsg();
-
 	register TScreen *screen = &term->screen;
 	int Xsocket = ConnectionNumber(screen->display);
 
@@ -1184,33 +1282,34 @@ spawn ()
 		setgid (screen->gid);
 		setuid (screen->uid);
 	} else {
-		Bool tty_got_hung = False;
+		Bool tty_got_hung;
 
- 		/*
- 		 * Sometimes /dev/tty hangs on open (as in the case of a pty
- 		 * that has gone away).  Simply make up some reasonable
- 		 * defaults.
- 		 */
+		/*
+		 * Sometimes /dev/tty hangs on open (as in the case of a pty
+		 * that has gone away).  Simply make up some reasonable
+		 * defaults.
+		 */
 
- 		signal(SIGALRM, hungtty);
- 		alarm(2);		/* alarm(1) might return too soon */
- 		if (! setjmp(env)) {
- 			tty = open ("/dev/tty", O_RDWR, 0);
- 			alarm(0);
- 		} else {
+		signal(SIGALRM, hungtty);
+		alarm(2);		/* alarm(1) might return too soon */
+		if (! setjmp(env)) {
+			tty = open ("/dev/tty", O_RDWR, 0);
+			alarm(0);
+			tty_got_hung = False;
+		} else {
 			tty_got_hung = True;
- 			tty = -1;
- 			errno = ENXIO;
- 		}
- 		signal(SIGALRM, SIG_DFL);
- 
+			tty = -1;
+			errno = ENXIO;
+		}
+		signal(SIGALRM, SIG_DFL);
+
 		/*
 		 * Check results and ignore current control terminal if
 		 * necessary.  ENXIO is what is normally returned if there is
 		 * no controlling terminal, but some systems (e.g. SunOS 4.0)
 		 * seem to return EIO.  Solaris 2.3 is said to return EINVAL.
 		 */
- 		if (tty < 0) {
+		if (tty < 0) {
 			if (tty_got_hung || errno == ENXIO || errno == EIO ||
 			    errno == EINVAL || errno == ENOTTY) {
 				no_dev_tty = TRUE;
@@ -1240,13 +1339,12 @@ spawn ()
 	}
 
 	/* avoid double MapWindow requests */
-	XtSetMappedWhenManaged( screen->TekEmu ? XtParent(tekWidget) :
-			        XtParent(term), False );
+	XtSetMappedWhenManaged(XtParent(CURRENT_EMU(screen)), False );
 
 	wm_delete_window = XInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW",
 				       False);
 
-	if (!screen->TekEmu)
+	if (!TEK4014_ACTIVE(screen))
 	    VTInit();		/* realize now so know window size for tty driver */
 
 	if (Console) {
@@ -1257,53 +1355,67 @@ spawn ()
 	    XmuGetHostname (mit_console_name + MIT_CONSOLE_LEN, 255);
 	    mit_console = XInternAtom(screen->display, mit_console_name, False);
 	    /* the user told us to be the console, so we can use CurrentTime */
-	    XtOwnSelection(screen->TekEmu ? XtParent(tekWidget) : XtParent(term),
+	    XtOwnSelection(XtParent(CURRENT_EMU(screen)),
 			   mit_console, CurrentTime,
 			   ConvertConsoleSelection, NULL, NULL);
 	}
+#if OPT_TEK4014
 	if(screen->TekEmu) {
 		envnew = tekterm;
 		ptr = newtc;
-	} else {
+	}
+	else
+#endif
+	{
 		envnew = vtterm;
 		ptr = termcap;
 	}
+
+	/*
+	 * This used to exit if no termcap entry was found for the specified
+	 * terminal name.  That's a little unfriendly, so instead we'll allow
+	 * the program to proceed (but not to set $TERMCAP) if the termcap
+	 * entry is not found.
+	 */
+	*ptr = 0;	/* initialize, in case we're using terminfo's tgetent */
 	TermName = NULL;
 	if (resource.term_name) {
+	    TermName = resource.term_name;
 	    if (tgetent (ptr, resource.term_name) == 1) {
-		TermName = resource.term_name;
-		if (!screen->TekEmu)
-		    resize (screen, TermName, termcap, newtc);
-	    } else {
-		fprintf (stderr, "%s:  invalid termcap entry \"%s\".\n",
-			 ProgramName, resource.term_name);
+		if (*ptr)
+		    if (!TEK4014_ACTIVE(screen))
+			resize (screen, termcap, newtc);
 	    }
 	}
 
+	/*
+	 * This block is invoked only if there was no terminal name specified
+	 * by the command-line option "-tn".
+	 */
 	if (!TermName) {
+	    TermName = *envnew;
 	    while (*envnew != NULL) {
 		if(tgetent(ptr, *envnew) == 1) {
 			TermName = *envnew;
-			if(!screen->TekEmu)
-			    resize(screen, TermName, termcap, newtc);
+			if (*ptr) 
+			    if(!TEK4014_ACTIVE(screen))
+				resize(screen, termcap, newtc);
 			break;
 		}
 		envnew++;
 	    }
-	    if (TermName == NULL) {
-		fprintf (stderr, "%s:  unable to find usable termcap entry.\n",
-			 ProgramName);
-		Exit (1);
-	    }
 	}
 
 	/* tell tty how big window is */
-	if(screen->TekEmu) {
+#if OPT_TEK4014
+	if(TEK4014_ACTIVE(screen)) {
 		ws.ws_row = 38;
 		ws.ws_col = 81;
 		ws.ws_xpixel = TFullWidth(screen);
 		ws.ws_ypixel = TFullHeight(screen);
-	} else {
+	} else
+#endif
+	{
 		ws.ws_row = screen->max_row + 1;
 		ws.ws_col = screen->max_col + 1;
 		ws.ws_xpixel = FullWidth(screen);
@@ -1351,11 +1463,12 @@ opencons();*/
 			 * (from ttyname)
 			 */
 #ifdef EMXNOTBOGUS
-			if (ptr = ttyname(tty)) {
+			if ((ptr = ttyname(tty)) != 0)
+			{
 				/* it may be bigger */
 				ttydev = realloc (ttydev, 
 					(unsigned) (strlen(ptr) + 1));
-				strcpy(ttydev, ptr);
+				(void) strcpy(ttydev, ptr);
 			}
 #else
 			ptr = ttydev;
@@ -1416,9 +1529,8 @@ opencons();*/
 			if(!TermName)
 				*newtc = 0;
 
-			sprintf (buf, "%lu", screen->TekEmu ? 
-				((unsigned long) XtWindow (XtParent(tekWidget))) :
-				((unsigned long) XtWindow (XtParent(term))));
+			sprintf (buf, "%lu",
+				((unsigned long) XtWindow (XtParent(CURRENT_EMU(screen)))));
 			Setenv ("WINDOWID=", buf);
 
 			/* put the display into the environment of the shell*/
@@ -1544,8 +1656,8 @@ opencons();*/
  */
 
 /*	signal (SIGINT, SIG_IGN);*/
-signal(SIGINT, killit);
-signal(SIGTERM, killit);
+	signal(SIGINT, killit);
+	signal(SIGTERM, killit);
 
 	/* hung shell problem */
 	signal (SIGQUIT, SIG_IGN);
@@ -1578,9 +1690,9 @@ Exit(n)
 }
 
 /* ARGSUSED */
-resize(screen, TermName, oldtc, newtc)
+static void
+resize(screen, oldtc, newtc)
 TScreen *screen;
-char *TermName;
 register char *oldtc, *newtc;
 {
 }
@@ -1601,16 +1713,16 @@ nonblocking_wait()
 
 /* ARGSUSED */
 static SIGNAL_T reapchild (n)
-    int n;
+    int n GCC_UNUSED;
 {
     int pid;
 
     pid = wait(NULL);
 
     /* cannot re-enable signal before waiting for child
-     * because then SVR4 loops.  Sigh.  HP-UX 9.01 too. 
+     * because then SVR4 loops.  Sigh.  HP-UX 9.01 too.
      */
-    signal(SIGCHLD, reapchild);
+    (void) signal(SIGCHLD, reapchild);
 
     do {
 	if (pid == term->screen.pid) {
@@ -1624,11 +1736,12 @@ static SIGNAL_T reapchild (n)
     SIGNAL_RETURN;
 }
 
+#if 0	/* this isn't used, but could be useful in debugging */
 /* VARARGS1 */
+void
 consolepr(fmt,x0,x1,x2,x3,x4,x5,x6,x7,x8,x9)
 char *fmt;
 {
-	extern char *SysErrorMsg();
 	int oerrno;
 	int f;
  	char buf[ BUFSIZ ];
@@ -1650,8 +1763,9 @@ char *fmt;
 	}
 #endif	/* TIOCNOTTY */
 }
+#endif
 
-
+static int
 remove_termcap_entry (buf, str)
     char *buf;
     char *str;
@@ -1715,14 +1829,12 @@ static int parse_tty_modes (s, modelist)
     }
 }
 
-
 int GetBytesAvailable (fd)
     int fd;
 {
-    static long arg;
+    long arg;
     ioctl (fd, FIONREAD, (char *) &arg);
     return (int) arg;
-
 }
 
 /* Utility function to try to hide system differences from
